import { useAuthenticated } from '@h/useAuthenticated';
import { logout } from '@q/session';
import { useLayoutStore } from '@st/layout_store';
import { useSessionStore } from '@st/session_store';
import { norNilOrEmpty } from '@u/funcs';
import { notification } from 'antd';
import dayjs from 'dayjs';
import { lte, not, range } from 'ramda';
import { FC, useCallback, useMemo } from 'react';
import { useNavigate } from 'react-router-dom';
import { useInterval } from 'react-use';
import 'twin.macro';
import { useAutoAsyncFn } from '@h/useAutoAsyncFn';

/**
 * 这是应用中的伺服精灵组件，不产生任何渲染输出，只用来负责控制应用的状态以及永久循环事件。
 */
export const Daemon: FC = () => {
  const isAuthenticated = useAuthenticated();
  const navigate = useNavigate();
  const [, logoutUser] = useAutoAsyncFn(logout);
  const warningDisplayed = useSessionStore.use.warningDisplayed();
  const showMask = useLayoutStore.use.showMask();
  const hideMask = useLayoutStore.use.hideMask();

  const unauthenticated = useMemo(() => {
    return not(isAuthenticated);
  }, [isAuthenticated]);

  const checkAndLogout = useCallback(() => {
    // 这里务必实时获取LocalStorage中保存的内容，如果依靠select函数，那么在各个计时中，数据将得不到变化。
    // ! 这里的内容在修改的时候，务必与useSession Hook中的LocalStorage使用方法、键名保持一致。
    const expiresAt = localStorage.getItem('expires') ?? '2000-01-01';
    const isLoggedInLocal = norNilOrEmpty(localStorage.getItem('token') ?? null);
    const isTokenExpiredLocal = dayjs(expiresAt).isBefore(dayjs());
    const difference = dayjs(expiresAt).diff(dayjs(), 'minute');

    [1, 5, 10].forEach((minutes, index) => {
      const warningDisplays = useSessionStore.getState().expireWarning;
      if (lte(difference, minutes) && not(warningDisplays[index])) {
        notification.warning({
          message: '登录会话即将结束',
          description: (
            <div>
              您的登录会话即将在<span tw="mx-2 text-xl text-red-6">{minutes}分钟</span>
              内结束，请尽快重新登录。
            </div>
          ),
          placement: 'bottomRight',
          duration: 10
        });
        range(index, 3).forEach(s => warningDisplayed(s));
      }
    });

    if (not(isLoggedInLocal) || isTokenExpiredLocal) {
      showMask();
      notification.warning({
        message: '登录会话结束',
        description: <div>您的登录会话已经结束，请重新登录。</div>,
        placement: 'bottomRight',
        duration: 5,
        onClose: async () => {
          await logoutUser();
          navigate('/login', { replace: true });
          hideMask();
        }
      });
    }
  }, []);

  useInterval(checkAndLogout, unauthenticated ? null : 30000);

  return null;
};
