//@ts-nocheck
import { useAuthenticated } from '@h/useAuthenticated';
import { useAuthorization } from '@h/useAuthorization';
import { isEmpty, mergeDeepLeft } from 'ramda';
import { Component, FunctionComponent, PropsWithChildren } from 'react';
import { Navigate, useLocation } from 'react-router-dom';

/**
 * 用于配置高阶组件`requireAuthorize`的认证及权限配置。
 *
 * 当配置项`all`被设置的时候，配置项`any`的内容将被忽略。如果不设置任何`all`或者`any`配置项，那么将仅判断用户是否已经登录。
 */
export interface AuthorizeOptions {
  /**
   * 配置指定组件要求用户必须拥有的全部权限。
   */
  all?: number[];

  /**
   * 配置指定组件要求用户必须至少拥有其中一项的权限。
   */
  any?: number[];

  /**
   * 设定当用户权限认证失败或者用户没有登录的时候，是否跳转到登录页面。
   */
  toLogin?: boolean;
}

/**
 * 对被包裹组件进行用户权限验证的组件。
 * ! 用户如果没有登录，那么验证将始终是失败的。
 *
 * @param param0 认证成功状态下路由转移到的组件以及所需要的权限声明
 * @returns 转向实际路由或者重定向到登录页面
 */
export const AuthRequired: FC<PropsWithChildren<AuthorizeOptions>> = ({
  all,
  any,
  toLogin,
  children
}: PropsWithChildren<AuthorizeOptions>) => {
  const requiredAllPrivileges = all ?? [];
  const requiredAnyPrivileges = any ?? [];
  const redirectToLogin = toLogin ?? false;
  const isAuthenticated = useAuthenticated();
  const { hasAll, hasAny } = useAuthorization();
  const location = useLocation();

  // 注意，这里的判断逻辑是：
  // 如果所需全部权限和任意权限都为空，即不需要任何权限，那么判断为通过；
  // 如果所需全部权限为空，那么取任意权限的判定；
  // 如果全部权限不为空，那么取全部权限的判断。
  const isUserMatchNeeds =
    (isEmpty(requiredAllPrivileges) && isEmpty(requiredAnyPrivileges)) ||
    (isEmpty(requiredAllPrivileges)
      ? hasAny(...requiredAnyPrivileges)
      : hasAll(...requiredAllPrivileges));

  if (isAuthenticated && isUserMatchNeeds) {
    return children;
  } else {
    return redirectToLogin ? (
      <Navigate to="/login" state={{ from: location }} replace={true} />
    ) : (
      <Navigate to="/unauthorized" state={{ from: location }} replace={true} />
    );
  }
};

/**
 * 动态用户权限验证的高阶组件。
 * ! 用户如果没有登录，那么验证将始终是失败的。
 *
 * @param Comp 需要被检验授权的组件
 * @param options 访问组件所需要的权限组合
 * @returns 新生成的具备动态判定权限的新组件
 */
export const requireAuthorize = (
  Comp: Component | FunctionComponent,
  options?: AuthorizeOptions
): FunctionComponent<any> => {
  const altedOptions = mergeDeepLeft(options, { any: [], all: [] });
  return props => {
    const isAuthenticated = useAuthenticated();
    const { hasAll, hasAny } = useAuthorization();
    const location = useLocation();

    const requiredAllPrivileges: string[] = altedOptions.all;
    const requiredAnyPrivileges: string[] = altedOptions.any;
    const redirectToLogin = altedOptions.toLogin ?? false;

    // 注意，这里的判断逻辑是：
    // 如果所需全部权限和任意权限都为空，即不需要任何权限，那么判断为通过；
    // 如果所需全部权限为空，那么取任意权限的判定；
    // 如果全部权限不为空，那么取全部权限的判断。
    const isUserMatchNeeds =
      (isEmpty(requiredAllPrivileges) && isEmpty(requiredAnyPrivileges)) ||
      (isEmpty(requiredAllPrivileges)
        ? hasAny(...requiredAnyPrivileges)
        : hasAll(...requiredAllPrivileges));

    if (isAuthenticated && isUserMatchNeeds) {
      return <Comp {...props} />;
    } else {
      return redirectToLogin ? (
        <Navigate to="/login" state={{ from: location }} replace={true} />
      ) : (
        <Navigate to="/unauthorized" state={{ from: location }} replace={true} />
      );
    }
  };
};

/**
 * 包裹需要根据用户所拥有的权限展示的组件。用户在满足给定的权限条件时，被包裹的组件方能被展示。
 * ! 用户如果没有登录，那么验证将始终是失败的。
 *
 * @param param0 认证成功状态下展示被包裹的组件所需要的权限定义
 * @returns 输出被包裹的组件，或者null
 */
export const WithAuth: FC<PropsWithChildren<AuthorizeOptions>> = ({
  all,
  any,
  children
}: PropsWithChildren<AuthorizeOptions>) => {
  const requiredAllPrivileges = all ?? [];
  const requiredAnyPrivileges = any ?? [];
  const isAuthenticated = useAuthenticated();
  const { hasAll, hasAny } = useAuthorization();

  // 注意，这里的判断逻辑是：
  // 如果所需全部权限和任意权限都为空，即不需要任何权限，那么判断为通过；
  // 如果所需全部权限为空，那么取任意权限的判定；
  // 如果全部权限不为空，那么取全部权限的判断。
  const isUserMatchNeeds =
    (isEmpty(requiredAllPrivileges) && isEmpty(requiredAnyPrivileges)) ||
    (isEmpty(requiredAllPrivileges)
      ? hasAny(...requiredAnyPrivileges)
      : hasAll(...requiredAllPrivileges));

  if (isAuthenticated && isUserMatchNeeds) {
    return children;
  }

  return null;
};
