//@ts-nocheck
import { omit } from 'ramda';
import { FC, PropsWithChildren } from 'react';
import tw, { styled, TwStyle } from 'twin.macro';

/**
 * 胶囊组件的类型，也即颜色搭配。
 */
type CapsuleType =
  | 'info'
  | 'warn'
  | 'error'
  | 'success'
  | 'primary'
  | 'plain'
  | 'gold'
  | 'silver'
  | 'green'
  | 'lime';

/**
 * 胶囊组件的尺寸。
 */
type CapsuleSize = 'nano' | 'mini' | 'small' | 'normal' | 'large' | 'extra';

const TypeStyles: { [key in CapsuleType]: TwStyle } = {
  info: tw`bg-blue-3 text-white`,
  warn: tw`bg-orange-6 text-white`,
  error: tw`bg-red-6 text-white`,
  success: tw`bg-green-4 text-white`,
  primary: tw`bg-sggreen-6 text-white`,
  plain: tw`bg-gray-4 text-white`,
  gold: tw`bg-gold-6 text-white`,
  silver: tw`bg-gray-6 text-white`,
  green: tw`bg-green-6 text-white`,
  lime: tw`bg-lime-6 text-white`
};

const HoverTypeStyles: { [key in CapsuleType]: TwStyle } = {
  info: tw`hover:bg-blue-3 hover:text-white`,
  warn: tw`hover:bg-orange-6 hover:text-white`,
  error: tw`hover:bg-red-6 hover:text-white`,
  success: tw`hover:bg-green-4 hover:text-white`,
  primary: tw`hover:bg-sggreen-6 hover:text-white`,
  plain: tw`hover:bg-gray-4 hover:text-white`,
  gold: tw`hover:bg-gold-6 hover:text-white`,
  silver: tw`hover:bg-gray-6 hover:text-white`,
  green: tw`hover:bg-green-6 hover:text-white`,
  lime: tw`hover:bg-lime-6 hover:text-white`
};

const ActiveTypeStyles: { [key in CapsuleType]: TwStyle } = {
  info: tw`active:text-white active:bg-blue-5 cursor-pointer`,
  warn: tw`active:text-white active:bg-orange-8 cursor-pointer`,
  error: tw`active:text-white active:bg-red-8 cursor-pointer`,
  success: tw`active:text-white active:bg-green-6 cursor-pointer`,
  primary: tw`active:text-white active:bg-sggreen-8 cursor-pointer`,
  plain: tw`active:text-white active:bg-gray-6 cursor-pointer`,
  gold: tw`active:text-white active:bg-gold-8 cursor-pointer`,
  silver: tw`active:text-white active:bg-gray-8 cursor-pointer`,
  green: tw`active:text-white active:bg-green-8 cursor-pointer`,
  lime: tw`active:text-white active:bg-lime-8 cursor-pointer`
};

const SizeStyles: Record<CapsuleSize, TwStyle> = {
  nano: tw`text-[0.35rem] leading-[0.5rem] px-0.5 py-0.5 rounded`,
  mini: tw`text-[0.5rem] leading-[0.75rem] px-1 py-0.5 rounded`,
  small: tw`text-xs px-1 py-0.5 rounded`,
  normal: tw`text-sm px-1.5 py-1 rounded-md`,
  large: tw`text-base px-1.5 py-1 rounded-md`,
  extra: tw`text-lg px-2 py-1.5 rounded-lg`
};

const CapsulePanel = styled.div(({ type, size, hover, activable }: Partial<CapsuleProps>) => [
  tw`select-none`,
  TypeStyles[type ?? 'plain'],
  SizeStyles[size ?? 'normal'],
  (hover ?? false) && HoverTypeStyles[hover ?? 'plain'],
  (activable ?? false) && (hover ?? false) && ActiveTypeStyles[hover ?? 'plain']
]);

/**
 * 胶囊组件的配置属性。
 */
export interface CapsuleProps {
  /**
   * 胶囊配色类型。
   */
  type: CapsuleType;

  /**
   * 配置胶囊是否可以响应鼠标悬停事件。
   */
  hover?: CapsuleType | boolean;

  /**
   * 配置胶囊是否可以响应鼠标按下事件，舒斌按下事件所使用的颜色与悬停颜色相同，并且受悬停影响。
   */
  activable?: boolean;

  /**
   * 点击胶囊时的回调函数，必须在胶囊设置了 `activable` 属性时才有效。
   */
  onClick?: (event: React.MouseEvent<HTMLDivElement, MouseEvent>) => void;

  /**
   * 胶囊大小。
   */
  size: CapsuleSize;

  /**
   * 胶囊中的文字内容。
   */
  content: string;
}

/**
 * 用于表示微型状态和提示信息的胶囊组件。
 * @param props 胶囊组件的配置属性
 * @returns 用于表示微型状态的胶囊组件。
 */
export const Capsule: FC<PropsWithChildren<CapsuleProps>> = (
  props: PropsWithChildren<CapsuleProps>
) => {
  const otherProps = omit(['type', 'hover', 'activable', 'size', 'content', 'onClick'], props);

  const handleClickAction = (event: React.MouseEvent<HTMLDivElement, MouseEvent>) => {
    (props.activable ?? false) && props.onClick && props.onClick(event);
  };

  return (
    <CapsulePanel
      type={props.type}
      size={props.size}
      hover={props.hover}
      activable={props.activable}
      onClick={handleClickAction}
      {...otherProps}
    >
      {props.content}
    </CapsulePanel>
  );
};
