import {useCallback, useEffect, useState} from 'react';
import { Region, regions, regionsDetail } from '@q/common';
import { isCorrectResult } from '@u/asyncs';
import { App, Cascader } from 'antd';
import { last, map } from 'ramda';
import { useQuery } from '@tanstack/react-query';

interface Option {
  value: string;
  label: string;
  children?: Option[];
  isLeaf?: boolean;
  loading?: boolean;
}

/**
 * 行政区划选择器
 */
export function SelectRegions(props: {
  // 外部应嵌套Form，初始化时Form传递的默认值为string，如果用户修改，则会变为array
  value?: string | string[];
  onChange?: (value: string[]) => void;
}) {
  const { message } = App.useApp();
  // 行政区划数据源
  const [options, setOptions] = useState<Option[]>([]);
  const [current, setCurrent] = useState<(string | 0)[]>([
    typeof props.value === 'string' ? 0 : props.value ? last(props.value) : 0
  ]);
  // 加载子级行政区划数据
  const onLoadRegion = useCallback(
    (selectedOptions?: Option[]) => {
      // 获取选择的最后的行政区
      const select = selectedOptions?.[selectedOptions.length - 1];
      setCurrent(selectedOptions.map(e => e.value));
      // 如果不是初次加载，则更新点击的项的loading状态
      if (select) select.loading = true;
    },
    [options]
  );
  useEffect(() => {

  }, []);
  // 手动触发时，加载逻辑
  useQuery(['select-region', ...current, props.value], () => regions(last(current)), {
    onSuccess: res => {
      if (isCorrectResult(res)) {
        setOptions(state => {
          if (last(current) === 0) return mapRegionsToOptions(res.regions);
          let target: Option;
          current.forEach(e => (target = (target?.children ?? state).find(o => o.value === e)));
          if (target) {
            target.children = mapRegionsToOptions(res.regions);
            target.loading = false;
          }
          return [...state];
        });
        return null;
      } else message.error(res.message ?? '加载行政区划失败！');
    }
  });

  // 初始选择的默认值，通过数组还是字符串来判断是否是表单自动填充的
  const { data: defaultValue, isFetching: defaultLoading } = useQuery(
    ['select-region-detail', props.value],
    () => regionsDetail(props.value as string),
    {
      enabled: typeof props.value === 'string' && !!props.value,
      select: res => {
        if (isCorrectResult(res)) {
          return {
            value: [props.value] as string[],
            label: res.regions
              .map(e => e.name)
              .reverse()
              .join('/')
          };
        } else {
          message.error(res.message ?? '加载行政区信息失败！');
          return null;
        }
      }
    }
  );

  return (
    <Cascader
      options={options}
      loadData={onLoadRegion}
      loading={defaultLoading}
      onChange={props.onChange}
      placeholder="请选择行政区划"
      value={typeof props.value === 'string' ? [defaultValue?.label] : props.value}
    />
  );
}

const mapRegionsToOptions = (regions: Region[]) =>
  map<Region, Option>(e => ({ value: e.code, label: e.name, loading: false, isLeaf: e.level === 3 }), regions);
