//@ts-nocheck
import { difference, equals, length, lt } from 'ramda';
import { useMemo } from 'react';
import { useSession } from './useSession';

/**
 * 用于根据LocalStorage中保存的会话信息判断当前用户是否是管理单位
 * @returns 当前用户是否是管理单位。
 */
export function useIsEnterprise() {
  const [user] = useSession();
  return equals(user.type, 0);
}

/**
 * 用于根据LocalStorage中保存的会话信息判断当用户是否是监管部门。
 * @returns 当前用户是否是监管部门。
 */
export function useIsSupervision() {
  const [user] = useSession();
  return equals(user.type, 1);
}

/**
 * 用于根据LocalStorage中保存的会话信息判断当用户是否是运维人员。
 * @returns 当前用户是否是运维人员。
 */
export function useIsOPS() {
  const [user] = useSession();
  return equals(user.type, 2);
}

/**
 * 用于判断当前已经登录的用户是否具备指定场景（权限）的Hook。
 * @returns 返回两个判断函数：hasAll()表示用户需要具有全部列出的权限，hasAny()表示用户需要具备列出的权限之一。
 */
export function useAuthorization() {
  const [user] = useSession();
  const ownedPrivileges = useMemo(() => [user?.type ?? -1], [user]);

  /**
   * 判断当前登录的用户是否具备所有要求拥有的权限。
   * @returns 是否全部具备要求拥有的权限
   * @param requires
   */
  function hasAll(...requires: number[]): boolean {
    const differs = difference(requires, ownedPrivileges);
    return equals(length(differs), 0);
  }

  /**
   * 判断当前登录的用户是否具备所要求的全部权限中的任意一个。
   * @param requires 要求拥有的权限
   * @returns 是否具备要求拥有的权限之一
   */
  function hasAny(...requires: number[]): boolean {
    const originalPrivilegesSize = length(requires);
    const differs = difference(requires, ownedPrivileges);
    return lt(length(differs), originalPrivilegesSize);
  }

  return { hasAll, hasAny };
}
