//@ts-nocheck
import { equals } from 'ramda';
import { useCallback, useRef } from 'react';
import { useMountedState } from 'react-use';
import { FunctionReturningPromise, PromiseType } from 'react-use/lib/misc/types';
import { AsyncFnReturn, AsyncState } from 'react-use/lib/useAsyncFn';
import { useImmer } from 'use-immer';
import { useAnswer403Forbidden } from './useAnswer403Forbidden';

type StateFromFunctionReturningPromise<T extends FunctionReturningPromise> = AsyncState<
  PromiseType<ReturnType<T>>
>;

export function useAutoAsyncFn<T extends FunctionReturningPromise>(
  fn: T,
  deps: DependencyList = [],
  initialState: StateFromFunctionReturningPromise<T> = { loading: false }
): AsyncFnReturn<T> {
  const [state, updateState] = useImmer<StateFromFunctionReturningPromise<T>>(initialState);
  const isMounted = useMountedState();
  const lastCallId = useRef(0);
  const { check: check403 } = useAnswer403Forbidden();

  const newCallback = useCallback(async (...args: Parameters<T>): Promise<ReturnType<T>> => {
    const callId = ++lastCallId.current;

    if (!state.loading) {
      updateState(draft => {
        draft.loading = true;
      });
    }

    try {
      const result = await fn(...args);
      if (isMounted() && equals(callId, lastCallId.current)) {
        updateState(draft => {
          draft.value = result;
          draft.loading = false;
        });
      }
      return result;
    } catch (e) {
      if (isMounted() && equals(callId, lastCallId.current)) {
        updateState(draft => {
          draft.error = e;
          draft.loading = false;
        });
      }
      check403(e);
      throw e;
    }
  }, deps);

  return [state, newCallback as unknown as T];
}
