//@ts-nocheck
import { SyncObjectCallback } from '@/shared/foundation';
import { reduceIndexed } from '@u/funcs';
import { append, defaultTo, gte, isEmpty, match, min, reduce, reduced, sum } from 'ramda';
import { useCallback } from 'react';

const detectRules = ['[a-z]', '[A-Z]', '[0-9]'];
const lengthStrengthRules = [0, 6, 8, 10];
const defaultAllowedSymbols = '!"#$%&\'()*+,-./:;<=>?@[\\\\\\]^_`{|}~';

/**
 * 返回一个用于检测密码强度的方法。返回的密码强度从0开始表示，0为极弱，4为强。
 * 密码强度计算根据密码中出现的字符种类和密码整体长度中较弱的一项确定。
 * @param allowedSymbols 用于指定密码中允许出现的除字母和数字以外的其他符号。不指定将使用默认的符号列表。
 * @returns 用于密码强度检测的方法。
 */
export function useCalculatePasswordStrength(
  allowedSymbols?: string
): SyncObjectCallback<string, number> {
  const finalAllowedSymbols = defaultTo(defaultAllowedSymbols)(allowedSymbols);

  const detector = useCallback(
    (pwd: string) => {
      const finalDetectRules = append(`[${finalAllowedSymbols}]`, detectRules);
      const rulesDetectResult = reduce(
        (acc, elem) => {
          const matches = match(elem, pwd);
          return append(isEmpty(matches) ? 0 : 1, acc);
        },
        [] as number[],
        finalDetectRules
      );
      const symbolStrengthLevel = sum(rulesDetectResult);
      const pwdLength = pwd.length;
      const lengthStrengthLevel = reduceIndexed<number, number>(
        (acc, elem, index) => (gte(pwdLength, elem) ? index : reduced(acc)),
        0,
        lengthStrengthRules
      );
      return min(symbolStrengthLevel, lengthStrengthLevel);
    },
    [finalAllowedSymbols]
  );

  return detector;
}
