import type { BaseResponse, Pageable } from '@/shared/model-components';
import { ajaxEngine } from '@q/axios_instance';
import { unwrap } from '@u/asyncs';
import { Record, Related, EditAccountingData, Settlement, BalanceOverall } from '@/shared/model-accounting';
import { downloadFile } from './common';
import { stringify } from 'qs';
import qs from 'qs';
import { UploadExcelError } from '@/shared/model-park';

/**
 * 查询账务记录
 * @param park 园区id
 * @param page 页码
 * @param keyword 关键词
 * @param  起始时间
 * @param periodEnd 截止时间
 * @param tenement 要查询的账务归属商户id
 */
export async function accountingList({
  page,
  keyword,
  periodStart,
  periodEnd,
  tenement,
  type,
  medium,
  park
}: {
  page: number,
  keyword?: string,
  periodStart?: string,
  periodEnd?: string,
  tenement?: string,
  type?:string,
  medium?: string,
    park?: string
}): Promise<BaseResponse & Pageable & { records: Record[], amount: number }> {
  const response = await ajaxEngine().get(`/accounting?${qs.stringify({
    page,
    keyword,
    periodStart,
    periodEnd,
    tenement,
    type,
    medium,
    park,
  })}`);
  return unwrap(response);
}

/**
 * 保存账务记录条目
 * @param data 信息
 */
export async function createAccounting(data: EditAccountingData): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/accounting`, data);
  return unwrap(response);
}


/**
 * 保存账务记录条目
 * @param id
 * @param data 信息
 */
export async function updateAccounting(id: string, data: EditAccountingData): Promise<BaseResponse> {
    const response = await ajaxEngine().put(`/accounting/${id}`, data);
    return unwrap(response);
}

/**
 * 回滚指定账务条目 删除操作
 * @param id
 */
export async function deleteAccounting(id: string): Promise<BaseResponse> {
    const response = await ajaxEngine().delete(`/accounting/${id}`);
    return unwrap(response);
}
/** 下载模板 */
export function download(pid: string) {
  downloadFile(`/meter/${pid}/template`);
}
/**
 * 获取授权信息（财务审核）
 */
export async function approve(data: { username: string, password: string, required: string [] }) {
  const response = await ajaxEngine().post(`/authorize`, data);
  return unwrap(response);
}

/** 查询结算记录 */
export async function settlementList(
  park: string,
  page: number,
  keyword?: string,
  settleStart?: string,
  settleEnd?: string,
  tenement?: string,
  status?: string
): Promise<BaseResponse & Pageable & { settlements: Settlement[] } > {
  const response = await ajaxEngine().get(`/settlement/${park}?${stringify({ page, keyword, settleStart, settleEnd, tenement, status } )}`, {

  });
  return unwrap(response);
}

/** 查询账务余额 */
export async function getAccountingBalance({
  parkId,
  tenement,
  shortName,
  page,
  code
}: {
  parkId: string,
  tenement?: string,
  shortName?: string,
  periodStart?: string,
  periodEnd?: string,
  page?: number,
  code?: string,
}) :Promise<BaseResponse & { related: Related[], overall: BalanceOverall }>{
  const response = await ajaxEngine().get(`/accounting/balancing`, {
    params: { page, tenement, code, parkId, shortName }
  });
  return unwrap(response);
}

// 获取商户下所有的表计
export async function getMetersByTenement(tenement: string) {
  const response = await ajaxEngine().get(`/accounting/Meter`, {
    params: { tenement,  }
  });
  return unwrap(response);
}

/**
 * 上传账务初始金额
 * @param pid 园区Id
 * @param file 文件对象
 * @param onProgress 进度监听
 */
export async function uploadAccountingBalanceFile(
  pid: string,
  file: File,
  onProgress?: (percent: number) => void
): Promise<BaseResponse & { errors: UploadExcelError[] | null }> {
  const response = await ajaxEngine().postForm(
    `/accounting/batch?pid=${pid}`,
    { data: file },
    { onUploadProgress: (e: ProgressEvent) => onProgress?.(e.loaded / e.total) }
  );
  return unwrap(response);
}

/** 余额导入模板 */
export function templateAccountingDownload(pid: string) {
  return downloadFile(`/accounting/template?park=${pid}`);
}