import type { BaseResponse, Pageable } from '@/shared/model-components';
import { ajaxEngine } from '@q/axios_instance';
import { unwrap } from '@u/asyncs';
import type {
  ReportInfo,
  ReportSteps,
  SummaryCaculateResult,
  ReportMaintenance,
  MeterRecord
} from '@/shared/model-calculate';
import type { MaintenanceFormData } from '@q/park';
import { downloadFile } from '@q/common';
import type { UploadExcelError } from '@/shared/model-park';
import { ReportFilled } from '@/shared/model-calculate';

/**
 * 列出当前所有园区最新的报表
 * @returns 报表
 */
export async function reportList(): Promise<BaseResponse & { parks: ReportInfo[] }> {
  const response = await ajaxEngine().get(`/reports/with/drafts`);
  return unwrap(response);
}

/**
 * 撤回公示
 * @param rid 核算记录Id
 */
export async function withdraw(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/withdraw/${rid}`);
  return unwrap(response);
}

/**
 * 初始化指定园区的报表
 * @param pid 园区Id
 * @param period 期数
 */
export async function initReport(pid: string, period: string): Promise<BaseResponse & { reportId: string }> {
  const response = await ajaxEngine().post(`/park/${pid}/report?period=${period}`);
  return unwrap(response);
}

/**
 * 获取指定报表的步骤状态
 * @param rid 报表Id
 */
export async function stepState(rid: string): Promise<BaseResponse & { steps: ReportSteps }> {
  const response = await ajaxEngine().get(`/report/${rid}/step/state`);
  return unwrap(response);
}

export interface SaveElectricBody {
  /** 园区ID */
  parkId: string;
  /** 核算起始时间 */
  periodBegin: string;
  /** 核算结束时间 */
  periodEnd: string;
  /** 线损率 */
  authorizedLossRate: string;
  /** 调整电费 */
  adjustFee: string;
  /** 基本电费 */
  basicFee: string;
  /** 有功(尖峰)电量 */
  critical: string;
  /** 有功(尖峰)电费 */
  criticalFee: string;
  /** 有功(总)电量，总电量 */
  overall: string;
  /** 有功(总)电费，总电费 */
  overallFee: string;
  /** 有功(峰)电量 */
  peak: string;
  /** 有功(峰)电费 */
  peakFee: string;
  /** 有功(谷)电量 */
  valley: string;
  /** 有功(谷)电费 */
  valleyFee: string;
  /** 光伏发电电量 */
  generateAmount: number;
}

/**
 * 试计算电量电费信息
 * @param data 需要提交的字段
 * @return 计算结果
 */
export async function calculateElectric(
  data: SaveElectricBody
): Promise<BaseResponse & { summary?: SummaryCaculateResult }> {
  const response = await ajaxEngine().post(`/report/calculate`, data);
  return unwrap(response);
}

/**
 * 计算并保存填写的园区电量，此操作会记录到数据库并更新步骤状态
 * @param data 需要提交的字段
 */
export async function saveElectric(data: SaveElectricBody): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/report`, data);
  return unwrap(response);
}

/**
 * 计算并更新填写的园区电量，此操作会记录到数据库并更新步骤状态
 * @param rid 报表Id
 * @param data 需要提交的字段
 */
export async function updateElectric(rid: string, data: SaveElectricBody): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/report/${rid}`, data);
  return unwrap(response);
}

/**
 * 列出目前报表中待摊薄的维护费
 * @param rid 报表Id
 */
export async function reportMaintenance(rid: string): Promise<BaseResponse & { fees: ReportMaintenance[] }> {
  const response = await ajaxEngine().get(`/report/${rid}/maintenance`);
  return unwrap(response);
}

/**
 * 删除指定维护费条目
 * @param rid 报表Id
 * @param mid 要删除的维护费ID
 */
export async function deleteMaintenance(rid: string, mid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/report/${rid}/maintenance/${mid}`);
  return unwrap(response);
}

/**
 * 从预置维护表中导入维护费项目
 * @param rid 报表Id
 */
export async function importMaintenance(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/report/${rid}/maintenance/import`);
  return unwrap(response);
}

/**
 * 更改待摊薄费用编辑步骤状态
 * @param rid 报表Id
 */
export async function saveMaintenance(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/report/${rid}/step/diluted/fees`);
  return unwrap(response);
}

/**
 * 增加一个临时维护费项目
 * @param rid 报表Id
 * @param data
 */
export async function createTempMaintenance(rid: string, data: MaintenanceFormData): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/report/${rid}/maintenance`, {
    ...data,
    period: data.period.format('YYYY')
  });
  return unwrap(response);
}

/**
 * 修改配电维护费的信息
 * @param rid 报表Id
 * @param mid 配电维护费Id
 * @param data
 */
export async function editMaintenance(
  rid: string,
  mid: string,
  data: { fee?: string; memo?: string; name?: string }
): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/report/${rid}/maintenance/${mid}`, data);
  return unwrap(response);
}

/**
 * 修改配电维护费的信息
 * @param rid 报表Id
 */
export function downloadMeterTemplate(rid: string) {
  downloadFile(`/report/${rid}/meter/template`);
}

/**
 * 导出计算信息
 * @param rid 报表Id
 * @param pid
 * @param periodBegin
 * @param periodEnd
 */
export async function downloadCalculate(rid: string, pid: string, periodBegin, periodEnd) {
  return downloadFile(`/report/export/${rid}/${pid}?periodBegin=${periodBegin}&periodEnd=${periodEnd}`);
}


/**
 * 上传户表抄表记录Excel
 * @param rid 报表Id
 * @param file 文件对象
 * @param onProgress 进度监听
 */
export async function uploadMeterRecordFile(
  rid: string,
  file: File,
  onProgress?: (percent: number) => void
): Promise<BaseResponse & { errors: UploadExcelError[] | null }> {
  const response = await ajaxEngine().postForm(
    `/report/${rid}/meter/batch`,
    { data: file },
    { onUploadProgress: (e: ProgressEvent) => onProgress?.(e.loaded / e.total) }
  );
  return unwrap(response);
}

/**
 * 获取报表中户表记录列表
 * @param rid 报表Id
 * @param page
 * @param keyword
 */
export async function meterRecords(
  rid: string,
  page: number,
  keyword: string
): Promise<BaseResponse & Pageable & { meters: MeterRecord[] }> {
  const response = await ajaxEngine().get(`/report/${rid}/submeter`, { params: { page, keyword } });
  return unwrap(response);
}

/**
 * 修改指定户表抄表记录
 * @param rid 报表Id
 * @param pid 园区Id
 * @param mid 表号
 * @param data
 */
export async function editMeterRecords(
  rid: string,
  pid: string,
  mid: string,
  data: Partial<MeterRecord>
): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/report/${rid}/submeter/${pid}/${mid}`, data);
  return unwrap(response);
}

/**
 * 更改抄表步骤状态
 * @param rid 报表Id
 */
export async function saveMeterStatus(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/report/${rid}/step/meter/register`);
  return unwrap(response);
}

/**
 * 对报表中已有的数据进行计算
 * @param rid 报表Id
 */
export async function tryCalculate(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/report/${rid}/calculate`);
  return unwrap(response);
}

/**
 * 发布指定的电费公示报表
 * @param rid 报表Id
 */
export async function publishReport(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/report/${rid}/publish`);
  return unwrap(response);
}

/**
 * 列举未发布报表接口
 */
export async function reportsDraft(
  user: string
): Promise<BaseResponse & Pageable & { reports: (Omit<ReportInfo, 'user'> & { message?: string })[] }> {
  const response = await ajaxEngine().get(`/report/draft`, { params: { user } });
  return unwrap(response);
}
/**
 * 综合列举报表接口
 * @param user
 * @param park
 * @param page
 * @param keyword
 * @param periodStart 报表核算起始日期，格式：yyyy-MM-dd，定义核算起始的时间，之后的都会列出。
 * @param periodEnd 报表核算的结束日期，格式：yyyy-MM-dd，定义核算最终结束的时间，之前的都会列出。
 * @param all
 */
export async function reports(
  user: string,
  park: string,
  page: number,
  keyword?: string,
  periodStart?: string,
  periodEnd?: string,
  all = 0
): Promise<BaseResponse & Pageable & { reports: ReportInfo[] }> {
  const response = await ajaxEngine().get(`/reports`, {
    params: { user, park, page, keyword, periodStart, periodEnd, all }
  });
  return unwrap(response);
}

/**
 * 执行核算报表的计算
 * @param rid 记录Id
 */
export async function calculate(rid: string) {
  return unwrap(await ajaxEngine().put(`/report/${rid}/calculate`));
}

/**
 * 发布指定的电费公示报表
 * @param rid 记录Id
 */
export async function publish(rid: string) {
  return unwrap(await ajaxEngine().post(`/report/${rid}/publish`));
}

/**
 * 获取已填写的园区电量信息
 */
export async function getReportFilled(rid: string): Promise<BaseResponse & { summary: ReportFilled }> {
  const response = await ajaxEngine().get(`/report/${rid}/summary/filled`);
  return unwrap(response);
}
