import type { BaseResponse, Pageable } from '@/shared/model-components';
import { ajaxEngine } from '@q/axios_instance';
import { unwrap } from '@u/asyncs';
import { ChargeInfo } from '@/shared/model-charge';
import qs from 'qs';

/**
 * 获取符合条件的商户充值记录
 * @param pid 园区id
 * @param page 页码
 * @param keyword 关键词
 * @param startDate 起始时间
 * @param endDate 截止时间
 * @param typeNumber
 */
export async function chargeList(
  pid: string,
  page: number,
  keyword?: string,
  startDate?: string,
  endDate?: string,
  typeNumber?: number
): Promise<BaseResponse & Pageable & { topUps: ChargeInfo[], amount: number }> {
  const response = await ajaxEngine().get(`/topup/${pid}?${qs.stringify({ page, keyword, startDate, endDate, type: typeNumber })}`);
  return unwrap(response);
}

/**
 * 删除一个充值记录
 * @param pid 园区Id
 * @param tid 充值记录Id
 */
export async function cancelCharge(pid: string, tid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/topup/${pid}/${tid}`);
  return unwrap(response);
}

/**
 * 获取用于下拉列表的商户列表
 * @param park 园区Id
 * @param keyword 检索关键字
 */
export async function tenementSearchChoice(
  park: string,
  keyword: string
): Promise<BaseResponse & { tenements: { fullName: string; id: string; park: string; shortName: string; parkName: string }[] }> {
  const response = await ajaxEngine().get('/tenement/choice', { params: { park, keyword, limit: 10 } });
  return unwrap(response);
}

/**
 * 获取用于下拉列表的商户列表
 * @param keyword 检索关键字
 */
export async function tenementSearchChoiceByName(
  keyword: string
): Promise<BaseResponse & { tenements: { fullName: string; id: string; park: string; shortName: string; parkName: string }[] }> {
  const response = await ajaxEngine().get('/tenement/choice', { params: { keyword, limit: 10 } });
  return unwrap(response);
}

/**
 * 获取用于下拉列表的商户列表，根据用户简称搜索
 * @param parkid
 * @param name
 */
export async function tenementSearchChoiceByShortName(
  parkid: string,
  name: string,
): Promise<BaseResponse & { tenements: { fullName: string; id: string; park: string; shortName: string }[] }> {
  const response = await ajaxEngine().get('/tenement/name', { params: { parkid, name, limit: 10 } });
  return unwrap(response);
}



/**
 * 创建一个新的充值记录
 * @param pid 园区id
 * @param data
 */
export async function createCharge(pid: string, data: { 
  amount: string; 
  meter: string; 
  tenement: string; 
  paymentType: number, 
  voucherNo: string, 
  type: "0" | "1" | "2",
  flowType: number,
  fileList?: string[]
}) {
  const response = await ajaxEngine().post(`/topup/${pid}`, data);
  return unwrap(response);
}

/**
 * 修改一个新的充值记录
 * @param pid 园区id
 * @param tid 记录id
 * @param data
 */
export async function updateCharge(tid: string, data: { 
  amount: string; 
  meter: string; 
  tenement: string; 
  paymentType: number, 
  voucherNo: string, 
  type: "0" | "1" | "2",
  flowType: number,
  fileList?: string[]
}) {
  const response = await ajaxEngine().put(`/topup/${tid}`, data);
  return unwrap(response);
}