import type { BaseResponse } from '@/shared/model-components';
import { ajaxEngine } from '@q/axios_instance';
import { unwrap } from '@u/asyncs';
import {alertError} from "@/utils";

/**
 * 行政区划
 */
export interface Region {
  /** 行政区划编号 */
  code: string;
  /** 行政区划级别 [0-3] */
  level: number;
  /** 行政区划名称 */
  name: string;
  /** 父级行政区划编号 */
  parent: string;
}

/**
 * 获取行政区划下的子级行政区划。
 * @param rid 父级行政区划编号，0：顶级行政区划
 */
export async function regions(rid: string | 0): Promise<BaseResponse & { regions: Region[] }> {
  const response = await ajaxEngine(true).get(`/region/${rid || 0}`);
  return unwrap(response);
}

/**
 * 获取指定行政区划所在的所有区划信息。
 * @param rid 行政区划编号
 */
export async function regionsDetail(rid: string): Promise<BaseResponse & { regions: Region[] }> {
  const response = await ajaxEngine(true).get(`/regions/${rid}`);
  return unwrap(response);
}

/**
 * 下载文件函数封装
 * @param url 文件地址
 */
export function downloadFile(url: string) {
  return fetch(ajaxEngine().defaults.baseURL + url, {
    headers: { ...ajaxEngine().defaults.headers.common, Accept: 'application/octet-stream' }
  })
    .then(async response => {
      const content = response.headers.get('content-disposition')

      if (content == null) {
        alertError("服务错误")
        return;
      }
      const fileName = content.match(/filename[^;=\n]*=((['"]).*?\2|[^;\n]*)/)[1];
      return { file: await response.blob(), name: decodeURI(fileName) };
    })
    .then(res => {
      const link = document.createElement('a');
      link.href = URL.createObjectURL(res.file);
      link.download = res.name;
      link.click();
      URL.revokeObjectURL(link.href);
    })
    .catch(error => {
      console.error('Error downloading file:', error);
    });
}

/**
 * 用户服务期限查询。
 */
export async function expiration(): Promise<BaseResponse & { expiration: string }> {
  const response = await ajaxEngine().get(`/expiration`);
  return unwrap(response);
}

/**
 * 待审核内容数量。
 */
export async function audits(): Promise<BaseResponse & { amounts: { withdraw: number } }> {
  const response = await ajaxEngine().get(`/audits`);
  return unwrap(response);
}

export interface StatisticResponse {
  // 企业用户数量
  enterprises: number;
  // 园区数量
  parks: number;
  reports: {
    // 园区ID
    id: string;
    // 园区名称
    name: string;
    // 最新期数
    period: null | string;
  }[];
}

/**
 * 统计当前系统中的报表
 */
export async function statistic(): Promise<BaseResponse & { statistics: StatisticResponse }> {
  const response = await ajaxEngine().get(`/stat/reports`);
  return unwrap(response);
}
/**
 * 获取系统中定义的基准核定线损率
 */
export async function normLossRate(): Promise<BaseResponse & { normAuthorizedLossRate: string }> {
  const response = await ajaxEngine().get(`/norm/authorized/loss/rate`);
  return unwrap(response);
}
/**
 * 获取系统中定义的基准核定税率
 */
export async function normLossTax(): Promise<BaseResponse & { normAuthorizedTaxRate: string }> {
  const response = await ajaxEngine().get(`/norm/authorized/tax`);
  return unwrap(response);
}
