import type { BaseResponse } from '@/shared/model-components';
import { ajaxEngine } from '@q/axios_instance';
import { unwrap } from '@u/asyncs';

/**
 * 请求重置指定报表的园区总计部分。
 * ! 天神模式方法，切勿随意调用。
 * @param rid 要做重置处理的报表ID
 * @returns 报表的重置操作处理结果。
 */
export async function resetReportSummary(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/report/${rid}/summary`);
  return unwrap(response);
}

/**
 * 请求删除指定报表的维护费部分。
 * ! 天神模式方法，切勿随意调用。
 * @param rid 要做重置处理的报表ID
 * @returns 报表的重置操作处理结果。
 */
export async function removeReportMaintenanceFees(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/report/${rid}/maintenance`);
  return unwrap(response);
}

/**
 * 请求重置指定报表的抄表记录部分。
 * ! 天神模式方法，切勿随意调用。
 * @param rid 要做重置处理的报表ID
 * @returns 报表的重置操作处理结果。
 */
export async function resetReportEndUserRegister(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/report/${rid}/meters`);
  return unwrap(response);
}

/**
 * 请求重新同步指定报表的抄表记录的基本档案。
 * ! 天神模式方法，切勿随意调用。
 * @param rid 要做重新同步处理的报表ID
 * @returns 报表的重新同步操作处理结果。
 */
export async function resynchronizeReportEndUserRegister(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/gm/report/${rid}/meters`);
  return unwrap(response);
}

/**
 * 请求重置指定报表的全部内容。
 * ! 天神模式方法，切勿随意调用。
 * @param rid 要做重置处理的报表ID
 * @returns 报表的重置操作处理结果。
 */
export async function resetReport(rid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/report/${rid}`);
  return unwrap(response);
}

/**
 * 请求强制删除指定报表。
 * ! 天神模式方法，切勿随意调用。
 * @param rid 要做删除处理的报表ID
 * @returns 报表的删除操作处理结果。
 */
export async function forceRemoveReport(pid: string, rid: string[]): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/report`, { params: { park: pid, reports: rid } });
  return unwrap(response);
}

/**
 * 请求删除指定园区中的指定配电维护费。
 * ! 天神模式方法，切勿随意调用。
 * @param pid 要做处理的园区ID
 * @param mid 要删除的指定维护费ID
 * @returns 园区配电维护费删除操作处理结果。
 */
export async function removeSpecificMaintenanceFeeInPark(pid: string, mid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/park/${pid}/maintenance/${mid}`);
  return unwrap(response);
}

/**
 * 请求删除指定园区中的全部配电维护费。
 * ! 天神模式方法，切勿随意调用。
 * @param id 要做处理的园区ID
 * @returns 园区配电维护费删除操作处理结果。
 */
export async function removeParkMaintenance(pid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/park/${pid}/maintenance`);
  return unwrap(response);
}

/**
 * 请求删除指定园区中的全部终端用户档案。
 * ! 天神模式方法，切勿随意调用。
 * @param id 要做处理的园区ID
 * @returns 园区终端用户档案删除操作处理结果。
 */
export async function removeParkMeters(pid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/park/${pid}/meters`);
  return unwrap(response);
}

/**
 * 请求强制删除指定园区。
 * ! 天神模式方法，切勿随意调用。
 * @param pid 要做删除处理的园区ID
 * @returns 园区的删除操作处理结果。
 */
export async function forceRemovePark(pid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/park/${pid}/force`);
  return unwrap(response);
}

/**
 * 请求强制删除指定企业用户。
 * ! 天神模式方法，切勿随意调用。
 * @param uid 要做删除处理的用户ID
 * @returns 用户的删除操作处理结果。
 */
export async function forceRemoveEnterprise(uid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/enterprise/${uid}/force`);
  return unwrap(response);
}

/**
 * 删除符合指定条件的表计公摊关系
 * ! 天神模式方法，切勿随意调用。
 */
export async function forceDelPoolMeters(park: string, meters: string[] = []): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/meter/pooling`, { params: { park, meters } });
  return unwrap(response);
}

/**
 * 删除与符合指定条件的商户绑定的表计关系
 * ! 天神模式方法，切勿随意调用。
 */
export async function forceDelTenementMeters(
  park: string,
  tenements: string[] = [],
  meters: string[] = []
): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/tenement/meter`, { params: { park, meters, tenements } });
  return unwrap(response);
}

/**
 * 删除符合给定条件的抄表记录
 * ! 天神模式方法，切勿随意调用。
 */
export async function forceDelReading(
  park: string,
  meters: string[] = [],
  read_at: string[] = []
): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/reading`, { params: { park, meters, read_at } });
  return unwrap(response);
}

/**
 * 删除符合给定条件的抄表记录
 * ! 天神模式方法，切勿随意调用。
 */
export async function forceDelTenements(park: string, tenements: string[] = []): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/tenement`, { params: { park, tenements } });
  return unwrap(response);
}

/**
 * 删除符合给定条件的抄表记录
 * ! 天神模式方法，切勿随意调用。
 */
export async function forceDelInvoices(
  park: string,
  invoices: string[] = [],
  tenements: string[] = []
): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/gm/invoice`, { params: { park, invoices, tenements } });
  return unwrap(response);
}
