import type { BaseResponse, Pageable } from '@/shared/model-components';
import { ajaxEngine } from '@q/axios_instance';
import { unwrap } from '@u/asyncs';
import {cover, InvoiceDetail, InvoiceInfo, TaxMethod, UninvoiceData} from '@/shared/model-invoice';
import qs from 'qs';
import { downloadFile } from './common';

/**
 * 获取已开具的发票记录列表
 * @param park
 * @param page 页码
 * @param keyword 关键词
 * @param startDate 起始时间
 * @param endDate 截止时间
 * @param tenement
 */
export async function invoiceList(
  park: string,
  page: number,
  keyword?: string,
  startDate?: string,
  endDate?: string,
  tenement?: string,
): Promise<BaseResponse & Pageable & { invoices: InvoiceInfo[] }> {
  const response = await ajaxEngine().get(`/invoice`, { params: { park, page, keyword, startDate, endDate, tenement } });
  return unwrap(response);
}

/**
 * 删除指定发票记录
 * @param code 发票号码
 */
export async function deleteInvoice(code: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/invoice/${code}`);
  return unwrap(response);
}

/**
 * 发票详情
 * @param code 发票编号
 */
export async function invoiceDetail(code: string): Promise<BaseResponse & { detail: InvoiceDetail }> {
  const response = await ajaxEngine().get(`/invoice/${code}`);
  return unwrap(response);
}

/**
 * 获取指定用户的未开票核算记录
 * @param tid 未开票核算信息
 */
export async function uninvoicedTenement({
  page,
  code,
  parkId,
  startDate,
  endDate,
  shortName,
  tenement
}: {
  page: number,
  code: string,
  parkId?: string,
  startDate: string,
  endDate: string,
  shortName: string,
  tenement: string,
}): Promise<BaseResponse & { records: UninvoiceData[], total: number }> {

  const response = await ajaxEngine().get(`/uninvoiced/tenement/report?${qs.stringify({ page, code, parkId, startDate, endDate, shortName, tenement })}`, );
  return unwrap(response);
}

export interface PrecalculateInvoiceData {
  /** 覆盖核算报表，列举已选择的核算报表ID */
  covers: cover[];
  /** 园区ID */
  park: string;
  /** 计税方式，0：核算电费含税，1：核算电费未税 */
  // taxMethod: TaxMethod;
  taxMethod: '0' | '1' | '';
  /** 税率 */
  // taxRate: number;
  taxRate: string;
  /** 商户ID */
  tenement: string;
  /** 要核算的表号 */
  // codes: string [];
}

/**
 * 试算发票的票面信息
 * @param data 预计算信息
 */
export async function precalculateInvoice(
  data: PrecalculateInvoiceData
): Promise<BaseResponse & { cargos: InvoiceDetail['cargos']; total: string }> {
  const response = await ajaxEngine().post(`/invoice/precalculate`, data);
  return unwrap(response);
}

export interface CreateInvoiceData {
  /** 覆盖核算报表，列举已选择的核算报表ID */
  covers: cover[];
  /** 发票号码 */
  invoiceNo: string;
  /** 发票类型 */
  invoiceType: string;
  /** 园区ID */
  park: string;
  /** 计税方式，0：核算电费含税，1：核算电费未税 */
  taxMethod: TaxMethod;
  /** 税率 */
  taxRate: number;
  /** 商户ID */
  tenement: string;
}

interface coverItem {
  rid: string,
  code: string,
}

export interface CreateInvoice {
  /** 覆盖核算报表，列举已选择的核算报表ID */
  covers: coverItem[];
  /** 发票号码 */
  invoiceNo: string;
  /** 发票类型 */
  invoiceType: string;
  /** 园区ID */
  park: string;
  /** 计税方式，0：核算电费含税，1：核算电费未税 */
  taxMethod: '0' | '1';
  /** 税率 */
  taxRate: string;
  /** 商户ID */
  tenement: string;
}

/**
 * 保存发票开具记录
 * @param data 信息
 */
export async function createInvoice(data: CreateInvoice): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/invoice`, data);
  return unwrap(response);
}

/**
 * 更新发票号
 */
export async function updateInvoiceNo(id: string, invoiceNo: string): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/invoice`, { "invoice_id": id, "invoice_no": invoiceNo});
  return unwrap(response);
}

/**
 * 已开发票
 */
export async function downloadInvoice(query) {
  return downloadFile(`/invoice/export?${qs.stringify(query)}`);
}