import type { BaseResponse, Pageable } from '@/shared/model-components';
import type {
  ElectrCategory,
  MaintenanceInfo,
  MaintenanceSummary,
  Meter04KvInfo,
  Meter04KvType, MeterBox,
  ParkInfo,
  UploadExcelError
} from '@/shared/model-park';
import {
  ExchangeFormData,
  MeterChoiceData,
  MeterPooled,
  MeterReading,
  MeterType,
  ParkBuilding,
  TenementInfo
} from '@/shared/model-park';
import { ajaxEngine } from '@q/axios_instance';
import { downloadFile } from '@q/common';
import { unwrap } from '@u/asyncs';
import { Dayjs } from 'dayjs';
import qs from 'qs';

/**
 * 请求园区列表
 * @param keyword 查询园区名称的关键字
 * @returns 园区列表
 */
export async function parkList(keyword: string): Promise<BaseResponse & { parks: ParkInfo[] }> {
  const response = await ajaxEngine().get(`/park`, {
    params: { keyword }
  });
  return unwrap(response);
}
/**
 * 请求指定用户的园区列表
 * @param uid 用户id
 * @returns 园区列表
 */
export async function userParkList(uid: string): Promise<BaseResponse & { parks: ParkInfo[] }> {
  const response = await ajaxEngine().get(`/park/belongs/${uid}`);
  return unwrap(response);
}

/**
 * 更改园区启用状态
 * @param pid 园区Id
 * @param enabled 是否启用
 */
export async function changeParkStatus(pid: string, enabled: boolean): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/park/${pid}/enabled`, { enabled });
  return unwrap(response);
}

/**
 * 删除园区
 * @param pid 园区Id
 */
export async function deletePark(pid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/park/${pid}`);
  return unwrap(response);
}

/**
 * 获取指定园区的信息
 * @param pid 园区Id
 */
export async function parkDetail(pid: string): Promise<BaseResponse & { park: ParkInfo }> {
  const response = await ajaxEngine().get(`/park/${pid}`);
  return unwrap(response);
}

/**
 * 获取指定用户下的所有园区
 * @param uid 用户Id
 */
export async function enterpriseParkList(uid: string): Promise<BaseResponse & { parks: ParkInfo[] }> {
  const response = await ajaxEngine().get(`/park/belongs/${uid}`);
  return unwrap(response);
}

export interface ChangeParkInfoParams {
  address?: string;
  area?: string;
  capacity?: string;
  category: ElectrCategory;
  contact?: string;
  name: string;
  phone?: string;
  region?: string;
  submeter: Meter04KvType;
  tenement?: string;
}

/**
 * 创建园区
 * @param data 创建的园区信息
 */
export async function createPark(data: Required<ChangeParkInfoParams>): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/park`, data);
  return unwrap(response);
}

/**
 * 修改园区信息
 * @param pid 园区Id
 * @param data 修改的园区信息
 */
export async function changeParkInfo(pid: string, data: ChangeParkInfoParams): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/park/${pid}`, data);
  return unwrap(response);
}

/**
 * 请求指定园区下的所有0.4kv表计
 * @param pid 园区id
 * @param page 页码
 * @param keyword 查询园区名称的关键字
 * @param meterType 类型
 * @returns 园区列表
 */
export async function meter04List(
  pid: string,
  page: number,
  keyword: string,
  meterType?: MeterType
): Promise<BaseResponse & Pageable & { meters: Meter04KvInfo[] }> {
  const response = await ajaxEngine().get(`/meter/${pid}`, { params: { page, keyword, type: meterType } });
  return unwrap(response);
}

/**
 * 下载0.4KV表计档案模板
 * @param pid 园区id
 */
export function template04Download(pid: string) {
  return downloadFile(`/meter/${pid}/template`);
}

/**
 * 创建0.4Kv表计
 * @param pid 园区Id
 * @param data 创建的0.4Kv表计信息
 */
export async function create04KvMeter(pid: string, data: Meter04KvInfo): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/meter/${pid}`, data);
  return unwrap(response);
}

/**
 * 修改0.4Kv表计信息
 * @param pid 园区Id
 * @param data 修改0.4Kv表计信息
 */
export async function change04KvMeterInfo(pid: string, data: Meter04KvInfo): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/meter/${pid}/${data.id}`, data);
  return unwrap(response);
}

/**
 * 换表
 * @param pid 园区Id
 * @param code 原表号
 * @param data 新表计信息
 */
export async function exchangeMeter(pid: string, code: string, data: ExchangeFormData): Promise<BaseResponse> {
  const response = await ajaxEngine().patch(`/meter/${pid}/${code}`, data);
  return unwrap(response);
}

/**
 * 获取指定0.4kv表计的详细信息
 * @param pid 园区Id
 * @param code 表号
 */
export async function getMeterInfo(pid: string, code: string): Promise<BaseResponse & { meter?: Meter04KvInfo }> {
  const response = await ajaxEngine().get(`/meter/${pid}/${code}`);
  return unwrap(response);
}
/**
 * 上传Excel文件
 * @param pid 园区Id
 * @param file 文件对象
 * @param onProgress 进度监听
 */
export async function upload04MeterFile(
  pid: string,
  file: File,
  onProgress?: (percent: number) => void
): Promise<BaseResponse & { errors: UploadExcelError[] | null }> {
  const response = await ajaxEngine().postForm(
    `/meter/${pid}/batch`,
    { data: file },
    { onUploadProgress: (e: ProgressEvent) => onProgress?.(e.loaded / e.total) }
  );
  return unwrap(response);
}

/**
 * 请求指定园区下的配电维护费
 * @param page 请求的页码
 * @param pid 园区id
 * @param period 请求检索的月份
 * @returns 维护费列表
 */
export async function maintenanceList(
  page: number,
  pid?: string,
  period?: string
): Promise<BaseResponse & Pageable & { fees: MaintenanceInfo[] }> {
  const response = await ajaxEngine().get(`/maintenance/fee`, {
    params: { page, park: pid, period }
  });
  return unwrap(response);
}

/**
 * 删除配电维护费项
 * @param mid 记录id
 */
export async function deleteMaintenance(mid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/maintenance/fee/${mid}`);
  return unwrap(response);
}

/**
 * 修改配电维护费的启用状态
 * @param mid 记录id
 * @param enabled 是否启用
 */
export async function changeMaintenance(mid: string, enabled: boolean): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/maintenance/fee/${mid}/enabled`, { enabled });
  return unwrap(response);
}

export interface MaintenanceFormData {
  fee: string;
  memo: string;
  name: string;
  period: Dayjs;
  parkId: string;
}

/**
 * 创建配电维护费
 * @param data 创建的配电维护费信息
 */
export async function createMaintenance(data: MaintenanceFormData): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/maintenance/fee`, {
    ...data,
    period: data.period.format('YYYY')
  });
  return unwrap(response);
}

/**
 * 修改配电维护费信息
 * @param mid 记录id
 * @param data 修改配电维护费信息
 */
export async function changeMaintenanceInfo(
  mid: string,
  data: { fee: string; memo: string; name: string }
): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/maintenance/fee/${mid}`, data);
  return unwrap(response);
}

/**
 * 从服务端获取物业附加费的统计
 * @param page 请求页码
 * @param uid 附加费所属的企业用户ID
 * @param pid 附加费所属的园区ID
 * @param period 附加费所属的年份
 * @param keyword 用于对用户信息和园区信息进行检索的关键字
 * @returns 从服务端获取到的物业附加费的统计。
 */
export async function retrieveAdditionalChargeStatistics(
  page: number,
  uid?: string,
  pid?: string,
  period?: string,
  keyword?: string
): Promise<BaseResponse & Pageable & { charges?: MaintenanceSummary[] }> {
  const response = await ajaxEngine().get('/additional/charges', {
    params: {
      page,
      user: uid ?? '',
      park: pid ?? '',
      period: period ?? '',
      keyword: keyword ?? ''
    }
  });
  return unwrap(response);
}

/**
 * 园区的建筑物列表
 * @param pid 园区id
 * @param keyword
 */
export async function getParkBuildings(pid: string, keyword?: string): Promise<BaseResponse & { buildings?: ParkBuilding[] }> {
  const response = await ajaxEngine().get(`/park/${pid}/building`, { params: { keyword } });
  return unwrap(response);
}

/**
 * 启用、禁用园区建筑物
 * @param pid 园区id
 * @param bid 建筑物id
 * @param enabled
 */
export async function changeBuildingStatus(pid: string, bid: string, enabled: boolean): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/park/${pid}/building/${bid}/enabled`, { enabled });
  return unwrap(response);
}

/**
 * 修改园区建筑信息
 * @param pid 园区id
 * @param bid 建筑物id
 * @param info
 */
export async function changeBuildingInfo(
  pid: string,
  bid: string,
  info: { name: string; floors: string }
): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/park/${pid}/building/${bid}`, info);
  return unwrap(response);
}

/**
 * 修改园区建筑信息
 * @param pid 园区id
 * @param info
 */
export async function createBuilding(pid: string, info: { name: string; floors: string }): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/park/${pid}/building`, info);
  return unwrap(response);
}

/**
 * 删除园区建筑物
 * @param pid 园区id
 * @param bid 建筑物id
 */
export async function deleteBuilding(pid: string, bid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/park/${pid}/building/${bid}`);
  return unwrap(response);
}

/**
 * 列出指定园区中的所有公摊表计
 * @param pid 园区id
 * @param page 页码
 * @param keyword 检索公摊表计的关键词
 * @returns 表计列表
 */
export async function meterPooledList(
  pid: string,
  page: number,
  keyword: string
): Promise<BaseResponse & Pageable & { meters: MeterPooled[] }> {
  const response = await ajaxEngine().get(`/meter/${pid}/pooled`, { params: { page, keyword } });
  return unwrap(response);
}

/**
 * 获取用于下拉列表的表计列表
 * @param park
 * @param keyword 检索公摊表计的关键词
 * @param limit
 * @returns 表计列表
 */
export async function pooledChoiceList(
  park: string,
  keyword: string,
  limit?: number
): Promise<BaseResponse & { meters: MeterChoiceData[] }> {
  const response = await ajaxEngine().get('/meter/choice', { params: { park, keyword, limit: limit || 6 } });
  return unwrap(response);
}

/**
 * 删除关联表计
 * @param pid 园区Id
 * @param mcode 要解除关联的父级表计编号
 * @param scode 要解除关联的子级表计编号
 */
export async function unBindMeter(pid: string, mcode: string, scode: string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/meter/${pid}/${mcode}/binding/${scode}`);
  return unwrap(response);
}

/**
 * 向分摊表计中绑定一个新的表计
 * @param pid 要绑定的表计所在园区id
 * @param code 要绑定的父级表计编号
 * @param meters 绑定的表计编号列表
 */
export async function bindMeters(pid: string, code: string, meters: string[]): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/meter/${pid}/${code}/binding`, meters);
  return unwrap(response);
}

/**
 * 列出园区中的商户
 * @param pid 园区id
 * @param page 页码
 * @param keyword 关键词
 * @param state 状态：0入住1迁出
 * @param building 选择的建筑
 * @param startDate
 * @param endDate 截止时间
 * @param code
 */
export async function tenementList(
  pid: string,
  page: number,
  state: 0 | 1,
  keyword?: string,
  building?: string,
  startDate?: string,
  endDate?: string,
  code?: string,
): Promise<BaseResponse & Pageable & { tenements: TenementInfo[] }> {
  const response = await ajaxEngine().get(`/tenement/${pid}`, {
    params: { page, keyword, building, startDate, endDate, state, code }
  });
  return unwrap(response);
}

/**
 * 列出指定商户下的所有表计
 * @param pid 园区id
 * @param tid 商户id
 * @param code
 */
// 2023-10-26 17:15，后端需要一个code参数，在商户查询绑定表计的时候传一个非空字符串，其他情况传空字符串
export async function tenementMeters(
  pid: string,
  tid: string,
  code?: string
): Promise<BaseResponse & Pageable & { meters: Meter04KvInfo[] }> {
  const response = await ajaxEngine().get(`/tenement/${pid}/${tid}/meter`,{ params: { code } });
  return unwrap(response);
}

/**
 * 解除商户与指定表计的关联
 * @param pid 园区Id
 * @param tid 商户Id
 * @param code 表计编号
 * @param data 解绑数据
 */
export async function unbindTenementMeter(
  pid: string,
  tid: string,
  code: string,
  data: MeterReading
): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/tenement/${pid}/${tid}/binding/${code}/unbind`, data);
  return unwrap(response);
}

/**
 * 迁出时每个表计数据
 */
export interface MoveOutFormData extends MeterReading {
  code: string;
  readAt: string;
}

/**
 * 迁出指定商户
 * @param pid 园区Id
 * @param tid 商户Id
 * @param data 迁出数据
 */
export async function moveOutTenement(pid: string, tid: string, data: MoveOutFormData[]): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/tenement/${pid}/${tid}/move/out`, data);
  return unwrap(response);
}

/**
 * 获取用于商户绑定下拉列表的表计列表
 * @param park 园区Id
 * @param keyword
 */
export async function unBindTenementMeters(park: string, keyword: string): Promise<BaseResponse & { meters: MeterChoiceData[] }> {
  const response = await ajaxEngine().get(`/meter/choice/tenement`, { params: { keyword, park } });
  return unwrap(response);
}

/**
 * 给商户绑定一个表计
 * @param pid 园区Id
 * @param tid 商户Id
 * @param data 绑定数据
 */
export async function bindTenementMeter(pid: string, tid: string, data: MoveOutFormData): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/tenement/${pid}/${tid}/binding`, data);
  return unwrap(response);
}

export interface CreateTenement {
  /** 开户账号，发票用 */
  account: string;
  /** 联系地址 */
  address: string;
  /** 开户行名称，发票用 */
  bank: string;
  /** 所在建筑ID */
  building: string;
  /** 联系人 */
  contact: string;
  /** 注册地址，发票用 */
  invoiceAddress: string;
  /** 注册电话，发票用 */
  invoicePhone: string;
  /** 商户的名称 */
  name: string;
  /** 所在建筑层数 */
  onFloor: string;
  /** 联系电话 */
  phone: string;
  /** 商户简称 */
  shortName: string;
  /** 社会统一信用代码，发票用 */
  usci: string;
  /** 入住日期 */
  moveIn: string;
}

/**
 * 增加一个商户
 * @param pid 园区Id
 * @param data 绑定数据
 */
export async function createTenement(pid: string, data: CreateTenement): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/tenement/${pid}`, data);
  return unwrap(response);
}

/**
 * 修改商户信息
 * @param pid 园区Id
 * @param tid 商户Id
 * @param data 修改数据
 */
export async function updateTenement(pid: string, tid: string, data: CreateTenement): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/tenement/${pid}/${tid}`, data);
  return unwrap(response);
}

export interface TenementInvoiceInfo {
  /** 开户账号 */
  account: string;
  /** 注册地址 */
  address: string;
  /** 开户行名称 */
  bank: string;
  /** 开票人 */
  name: string;
  /** 注册电话 */
  phone: string;
  /** 开票人社会统一信用代码 */
  usci: string;
}

/**
 * 获取指定商户的详细信息
 * @param pid 园区id
 * @param tid 商户id
 */
export async function tenementDetail(
  pid: string,
  tid: string
): Promise<BaseResponse & { tenement: TenementInfo & { invoiceInfo?: TenementInvoiceInfo } }> {
  const response = await ajaxEngine().get(`/tenement/${pid}/${tid}`);
  return unwrap(response);
}
/**
 * 导出表计
 */
export async function downloadMeterRecords(query) {
  return downloadFile(`/meter/export?${qs.stringify(query)}`);
}

/**
 * 导出商户
 */
export async function downloadTenementRecords(query) {
  return downloadFile(`/tenement/export?${qs.stringify(query)}`);
}

/**
 * 导出商户表计关系
 */
export async function downloadTenementMeterRecords(query) {
  return downloadFile(`/tenementMeter/export?${qs.stringify(query)}`);
}
/**
 * 创建电表箱
 */
export async function createMeterBox(data: MeterBox): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/box`, data);
  return unwrap(response);
}
/**
 * 修改电表箱
 */
export async function updateMeterBox(data: MeterBox): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/box/batch`, data);
  return unwrap(response);
}

/**
 * 删除电表箱
 */
export async function deleteMeterBox(id:string): Promise<BaseResponse> {
  const response = await ajaxEngine().delete(`/box/${id}`);
  return unwrap(response);
}
/**
 * 查询电表箱
 */
export async function getMeterBoxList(pid:string, page: number, keyword: string, building: string): Promise<BaseResponse & { data: MeterBox[] }> {
  const response = await ajaxEngine().get(`/box/${pid}`,{ params: {page, keyword, building} });
  return unwrap(response);
}
