import { Consumption } from '@/shared/model-calculate';
import type { BaseResponse, Pageable } from '@/shared/model-components';
import type { BasePublicityInfo, PublicityEndUserPeriodStatistics, ReportTenement, errorMessageItem } from '@/shared/model-publicity';
import { ParkSummary, PublicPooled, ReportTenementDetail } from '@/shared/model-publicity';
import { ajaxEngine } from '@q/axios_instance';
import { unwrap } from '@u/asyncs';
import {downloadFile} from "@q/common";

/**
 * 请求审核列表
 * @param page
 * @param keyword 查询的关键字
 * @returns 审核列表
 */
export async function auditList(
  page: number,
  keyword: string
): Promise<BaseResponse & Pageable & { records: BasePublicityInfo[] }> {
  const response = await ajaxEngine().get(`/withdraw`, { params: { keyword } });
  return unwrap(response);
}

/**
 * 通过/不通过审核
 * @param rid 审核记录Id
 * @param audit 审核操作
 */
export async function auditReport(rid: string, audit: boolean): Promise<BaseResponse> {
  const response = await ajaxEngine().put(`/withdraw/${rid}`, { audit });
  return unwrap(response);
}

/**
 * 获取指定报表的全部内容
 * @param rid 报表Id
 */
export async function reportDetail(
  rid: string
): Promise<BaseResponse & { detail: Omit<BasePublicityInfo, 'user'> & { enterprise: BasePublicityInfo['user'] } }> {
  const response = await ajaxEngine().get(`/report/${rid}`);
  return unwrap(response);
}

/**
 * 获取指定报表的园区概况
 * @param rid 报表Id
 */
export async function reportParkDetail(rid: string): Promise<BaseResponse & { summary: ParkSummary }> {
  const response = await ajaxEngine().get(`/report/${rid}/summary`);
  return unwrap(response);
}

/**
 * 获取公共表计电费情况
 * @param rid 报表Id
 * @param page
 * @param keyword
 */
export async function reportPublicFee(
  rid: string,
  page: number,
  keyword?: string
): Promise<BaseResponse & Pageable & { public: (PublicPooled & { adjustLoss: Consumption })[] }> {
  const response = await ajaxEngine().get(`/report/${rid}/publics`, { params: { page, keyword } });
  return unwrap(response);
}

/**
 * 获取公摊表计电费情况
 * @param rid 报表Id
 * @param page
 * @param keyword
 */
export async function reportPublicPooled(
  rid: string,
  page: number,
  keyword?: string
): Promise<BaseResponse & Pageable & { pooled: PublicPooled[] }> {
  const response = await ajaxEngine().get(`/report/${rid}/pooled`, { params: { page, keyword } });
  return unwrap(response);
}

/**
 * 获取指定公摊表计下的摊薄表计详细
 * @param rid 报表Id
 * @param code 表号
 */
export async function reportPooledSubmeter(
  rid: string,
  code: string
): Promise<BaseResponse & { meters: PublicPooled[] }> {
  const response = await ajaxEngine().get(`/report/${rid}/pooled/${code}/submeter`);
  return unwrap(response);
}

/**
 * 获取指定商户的核算详细
 * @param rid 报表Id
 * @param tid 商户Id
 */
export async function reportTenementDetail(
  rid: string,
  tid: string
): Promise<BaseResponse & { detail: ReportTenementDetail, amount: string }> {
  const response = await ajaxEngine().get(`/report/${rid}/tenement/${tid}`);
  return unwrap(response);
}

/**
 * 获取商户电量电费列表
 * @param rid 报表Id
 * @param page 页数
 * @param keyword 关键字
 */
export async function reportTenement(
  rid: string,
  page: number,
  keyword?: string
): Promise<BaseResponse & Pageable & { tenements: ReportTenement[] }> {
  const response = await ajaxEngine().get(`/report/${rid}/tenement`, { params: { page, keyword } });
  return unwrap(response);
}

/**
 * 综合列举报表接口
 * @param page 页码
 * @param keyword 关键字
 * @param period 期数
 * @param park 园区
 * @param user 管理单位
 */
export async function reportList(
  page: number,
  keyword: string,
  user: string,
  park: string,
  period: string
): Promise<BaseResponse & Pageable & { reports: BasePublicityInfo[] }> {
  const response = await ajaxEngine().get(`/reports`, {
    params: { page, keyword, user, park, periodStart: period, periodEnd: period, all: 1 }
  });
  return unwrap(response);
}

/**
 * 从服务端查询一段时间之内的终端用户调整费用统计。
 * @param uid 指定要查询的企业用户ID
 * @param pid 指定要查询的园区ID
 * @param start 统计开始日期，格式：YYYY-MM，指定日期包含
 * @param end 统计结束的日期，格式：YYYY-MM，指定日期包含
 */
export async function retrieveEndUserStat(
  uid?: string,
  pid?: string,
  start?: string,
  end?: string
): Promise<BaseResponse & { details?: PublicityEndUserPeriodStatistics[] }> {
  const response = await ajaxEngine().get(`/end/user/adjusts`, {
    params: {
      user: uid ?? '',
      park: pid ?? '',
      start: start ?? '',
      end: end ?? ''
    }
  });
  return unwrap(response);
}


/**
 * 获取报表核算的错误列表（如果没有返回的是空列表）
 * @param rid 报表id
 * */
export async function getErrorList(
  rid: string,
): Promise<BaseResponse & {errType: number} & { message: errorMessageItem[] }> {
  const response = await ajaxEngine().get(`/report/error/list?rid=${rid}`);
  return unwrap(response);
}

/**
 * 导出计算信息
 * @param rid 报表Id
 * @param pid
 */
export async function downloadReport(rid: string, pid: string) {
  return downloadFile(`/report/export/user/${rid}/${pid}`);
}
