import type { BaseResponse, Pageable } from '@/shared/model-components';
import { ajaxEngine } from '@q/axios_instance';
import { unwrap } from '@u/asyncs';
import {ReadingInfo, Refund, RefundForm} from '@/shared/model-reading';
import { MeterChoiceData, MeterType, UploadExcelError } from '@/shared/model-park';
import { downloadFile } from '@q/common';
import qs from "qs";

/**
 * 查询符合指定条件的抄表记录
 * @param pid 园区id
 * @param page 页码
 * @param keyword 关键词
 * @param building 选择的建筑
 * @param startDate 开始时间
 * @param endDate 截止时间
 * @param type
 */
export async function readingList(
  pid: string,
  page: number,
  keyword?: string,
  building?: string,
  startDate?: string,
  endDate?: string,
  type: MeterType = undefined
): Promise<BaseResponse & Pageable & { records: ReadingInfo[] }> {
  const response = await ajaxEngine().get(`/reading/${pid}`, {
    params: { page, keyword, building, startDate, endDate, type }
  });
  return unwrap(response);
}

/**
 * 获取用于增加抄表记录时下拉列表的表计列表
 * @returns 表计列表
 * @param park
 * @param keyword
 */
export async function meterChoiceList(
  park: string,
  keyword: string
): Promise<BaseResponse & { meters: MeterChoiceData[] }> {
  const response = await ajaxEngine().get(`/reading/${park}/choice`, { params: { keyword, limit: 10 } });
  return unwrap(response);
}

export type CreateReadingData = ReadingInfo & { code: string; readAt: string };

/**
 * 新建一条抄表记录
 * @param pid 园区id
 * @param data
 */
export async function createReading(pid: string, data: CreateReadingData) {
  const response = await ajaxEngine().post(`/reading/${pid}/${data.code}`, data);
  return unwrap(response);
}

/**
 * 新建一条抄表记录
 * @param pid 园区id
 * @param readAtTimestamp 抄表记录的时间戳
 * @param data
 */
export async function updateReading(pid: string, readAtTimestamp: string, data: CreateReadingData) {
  const response = await ajaxEngine().put(`/reading/${pid}/${data.code}/${readAtTimestamp}`, data);
  return unwrap(response);
}

/**
 * 下载抄表记录批量填写模板
 * @param pid 园区id
 */
export function downloadReadingTemplate(pid: string) {
  downloadFile(`/reading/${pid}/template`);
}

/**
 * 上传批量抄表记录档案
 * @param pid 园区id
 * @param file 文件对象
 * @param onProgress 进度监听
 */
export async function uploadReadingTemplateFile(
  pid: string,
  file: File,
  onProgress?: (percent: number) => void
): Promise<BaseResponse & { errors: UploadExcelError[] | null }> {
  const response = await ajaxEngine().postForm(
    `/reading/${pid}/batch`,
    { data: file },
    { onUploadProgress: (e: ProgressEvent) => onProgress?.(e.loaded / e.total) }
  );
  return unwrap(response);
}

/**
 * 获取退补电费列表
 * @param park
 * @param page
 * @param startDate
 * @param endDate
 * @param keyword
 */
export async function getRefundList(page: number, park?:string,  startDate?:string, endDate?:string, keyword?: string): Promise<BaseResponse & {data: Refund[], total: number}> {
  const response = await ajaxEngine().get(`/read/${park}/electric?${qs.stringify({
    page,
    park,
    start: startDate,
    end: endDate,
    keyword
  })}`, );
  return unwrap(response);
}
/**
 * 获取退补电费列表
 * @param park
 * @param data
 */
export async function createRefund(park: string, data: RefundForm): Promise<BaseResponse & {data: Refund[]}> {
  const response = await ajaxEngine().post(`/read/${park}/electric`, data);
  return unwrap(response);
}

/**
 * 查询退补电量可选周期
 * @param park
 * @param year 某个年份不能选
 */
export async function getDisabledMonth(park: string, year: number): Promise<BaseResponse & {data: string[]}> {
  const response = await ajaxEngine().get(`/read/${park}/period`, {
    params: {
      year
    }
  });
  return unwrap(response);
}

/**
 * 删除指定的电费退补记录
 * */
export async function deleteRefund(id: string) {
  const response = await ajaxEngine().delete(`/read/${id}`)
  return unwrap(response)
}
