import type { BaseResponse, Pageable } from '@/shared/model-components';
import { ajaxEngine } from '@q/axios_instance';
import { unwrap } from '@u/asyncs';
import {ParkSync, ReadingInterval, ReadingRetryAlgorithm, ReadingType, SyncTask} from '@/shared/model-sync';

/**
 * 同步列表
 * @param page
 * @param keyword 关键词
 */
export async function syncList(
  page: number,
  keyword?: string,
): Promise<BaseResponse & Pageable & { tasks: SyncTask[] }> {
  const response = await ajaxEngine().get(`/synchronize/task`, { params: {  keyword, page } });
  return unwrap(response);
}

/**
 * 暂停同步任务
 */
export async function pauseSyncTask(pid: string, tid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/synchronize/task/${pid}/${tid}/pause`);
  return unwrap(response);
}

/**
 * 将指定任务列入执行队列
 */
export async function startSyncTask(pid: string, tid: string): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/synchronize/task/${pid}/${tid}/enqueue`);
  return unwrap(response);
}

export interface SyncSettingData {
  /** 企业ID */
  entId: string;
  /** 采集系统型号 */
  imrs: string;
  /** 同步登录账号 */
  imrsAccount: string;
  /** 同步登录私钥，Base64或者私钥文件内容 */
  imrsKey: string;
  /** 同步登录密钥，加盐双向加密 */
  imrsSecret: string;
  /** 采集周期，0：每小时，1：每日，2：每周，3：每月 */
  interval: ReadingInterval;
  /** 最大重试次数 */
  maxRetries: string;
  /** 园区ID */
  parkId: string;
  /** 采集方式，0：自动+人工，1：自动，2：人工 */
  readingType: ReadingType;
  /** 重试间隔算法，0：指数退避，1：2倍线性间隔，2：3倍线性间隔，3：固定间隔 */
  retryAlgorithm: ReadingRetryAlgorithm;
  /** 重试间隔，基础间隔时间，根据间隔算法不同会产生不同的间隔 */
  retryInterval: string;
  /** 采集时间，格式：HH:mm */
  collectAt: string;
}

/**
 * 更新指定用户的同步配置
 */
export async function changeSyncSetting(uid: string, data: SyncSettingData): Promise<BaseResponse> {
  const response = await ajaxEngine().post(`/synchronize/configuration`, data);
  return unwrap(response);
}

/**
 * 获取用户的同步配置
 */
export async function getSyncSetting(pid: string, uid: string): Promise<BaseResponse & { setup: SyncSettingData }> {
  const response = await ajaxEngine().get(`/synchronize/configuration`, { params: { user: uid, park: pid } });
  return unwrap(response);
}

/**
 * 获取某个园区充值记录校验
 */
export async function getSyncRechargeRecords(pid: string, pageNum: number): Promise<BaseResponse & { data: ParkSync[], total: number }> {
  const response = await ajaxEngine().get(`/synchronize/task/${pid}/getRechargeRecords?pageNum=${pageNum}`,);
  return unwrap(response);
}
/**
 * 获取某个园区充值记录校验
 */
export async function resetSync(id: string): Promise<BaseResponse & { setup: SyncSettingData }> {
  const response = await ajaxEngine().get(`/synchronize/task/${id}/resetSync`,);
  return unwrap(response);
}

