/**
 * 用于描述从后端传来的使用code承载内容编号，name承载内容名称的基本数据结构。
 */
export interface BaseContent {
  /**
   * 内容编号。
   */
  code: string;

  /**
   * 内容名称。
   */
  name: string;
}

/**
 * 用于解析来自服务端的响应体基本内容，其他响应可直接继承和扩展本接口
 */
export interface BaseResponse {
  /**
   * 用于记录服务端处理状态码
   */
  code: number;

  /**
   * 用于记录服务端处理状态信息
   */
  message: string;
}

/**
 * 向服务端查询数量的响应体。
 */
export interface AmountResponse extends BaseResponse {
  /**
   * 所查询的内容数量。
   */
  amount: number;
}

/**
 * 用于记录条目修改时间以及修改人的记录。
 */
export interface ModificationRecords {
  /**
   * 记录条目创建时间。
   */
  createdAt: Date | string | number | null;

  /**
   * 记录条目创建人名称或代号。
   */
  createdBy: string | number | null;

  /**
   * 记录条目修改时间。
   */
  modifiedAt: Date | string | number | null;

  /**
   * 记录条目修改人名称或代号。
   */
  modifiedBy: string | number | null;

  /**
   * 记录条目删除时间，如果未删除则为null，可以用于实现软删除。
   */
  deletedAt: Date | string | number | null;

  /**
   * 记录条目删除人名称或代号。
   */
  deletedBy: string | number | null;

  /**
   * 记录条目撤销时间，注意名词与删除和取消的区别。
   */
  revokedAt: Date | string | number | null;

  /**
   * 记录条目撤销人名称或代号。
   */
  revokedBy: string | number | null;

  /**
   * 记录条目取消时间，注意名词与删除和撤销的区别。
   */
  canceledAt: Date | string | number | null;

  /**
   * 记录条目取消人名称或代号。
   */
  canceledBy: string | number | null;
}

/**
 * 定义解析分页内容所需要的响应参数。
 */
export interface Pageable {
  /**
   * 当前所返回的数据所在的页面。
   */
  current: number;

  /**
   * 当前所返回的数据每页条目数量。
   */
  pageSize: number;

  /**
   * 当前所有的页面总数。
   */
  total: number;
}

/**
 * 定义可以支持模型数据进行排序的类型。
 */
export interface Sortable {
  /**
   * 排序字段。
   */
  sort: number;
}

/**
 * 定义记录条目的字符串型状态信息。
 */
export interface Stateful {
  /**
   * 记录条目的状态信息。
   */
  state: string | boolean;
}

/**
 * 用于描述事物的修订版本信息的类型。
 */
export interface Revision extends Partial<ModificationRecords> {
  /**
   * 修订版本号。
   */
  revision: number;
}

/**
 * 用于在事物中记录当前活跃修订版本号以及可用的修订版本列表的类型。
 */
export interface Revisionary {
  /**
   * 当前活跃的修订版本。
   */
  revision: number;

  /**
   * 可用的全部修订版本列表。（可选）
   */
  revisions: Revision[];
}

/**
 * 用于记录条目的所有者信息。
 */
export type HasOwner = {
  [Property in keyof BaseContent as `owner${Capitalize<string & Property>}`]:
    | BaseContent[Property]
    | null;
};

/**
 * 用于记录条目中的简易行政区划描述信息。
 * 这里所给出的行政区划信息是多级行政区划合成以后的。
 */
export type SimpleAreaDescription = {
  [Property in keyof BaseContent as `area${Capitalize<string & Property>}`]:
    | BaseContent[Property]
    | null;
};
