import type { BaseUserInfo } from './model-user';

/**
 * 园区信息
 */
export type ParkInfo = SimpleParkInfo & {
  /** 条目创建时间，时间格式为 yyyy-MM-dd HH:mm:ss */
  createdAt: string;
  /** 条目删除时间，时间格式为 yyyy-MM-dd HH:mm:ss */
  deletedAt: null | string;
  /** 条目是否启用 */
  enabled: boolean;
  /** 发票税率，0-1的小数 */
  taxRate?: string;
  /** 基准核定线损率，0-1的小数 */
  normAuthorizedLoss?: string;
  /** 条目最后修改时间，时间格式为 yyyy-MM-dd HH:mm:ss */
  lastModifiedAt: null | string;
  /** 电价计算政策，0：园区平均电度电价，1：园区平均电价 */
  pricePolicy: PricePolicyType;
  /** 公摊电费分摊策略，0：不分摊，1：按电量分摊，2：按面积分摊 */
  publicDiluted: DilutedType;
  /** 线损电费分摊策略，0：不分摊，1：按电量分摊，2：按面积分摊 */
  lossDiluted: DilutedType;
  /** 基本电费分摊策略，0：不分摊，1：按电量分摊，2：按面积分摊 */
  basisDiluted: DilutedType;
  /** 调整电费分摊策略，0：不分摊，1：按电量分摊，2：按面积分摊 */
  adjustDiluted: DilutedType;
  voltageTransmissionPrice?: string,
};

/**
 * 基础园区信息
 */
export interface SimpleParkInfo {
  /** 园区ID */
  id: string;
  /** 园区所属用户ID */
  userId: string;
  /** 园区名称 */
  name: string;
  /** 园区住户数量 */
  tenement: null | string;
  /** 园区面积 */
  area: null | string;
  /** 供电容量 */
  capacity: null | string;
  /** 用电分类，0：两部制，1：单一峰谷，2：单一单一 */
  category: ElectrCategory;
  /** 户表计量类型，0：非峰谷，1：峰谷 */
  meter04kvType: Meter04KvType;
  /** 园区所在行政区划 */
  region: null | string;
  /** 园区地址 */
  address: null | string;
  /** 园区联系人 */
  contact: null | string;
  /** 园区联系人电话 */
  phone: null | string;
}

/**
 * 用电类别
 */
export const enum ElectrCategory {
  /** 两部制 */
  DoubleCompose,
  /** 单一峰谷 */
  PeakToValley,
  /** 单一电价 */
  Single
}

/**
 * 户表计量类型
 */
export const enum Meter04KvType {
  /** 非峰谷 */
  DisableValley,
  /** 峰谷 */
  Valley
}

/**
 * 用户电价政策
 */
export const enum PricePolicyType {
  /** 平均电度电价 */
  Unit,
  /** 平均电价 */
  Avg
}

/**
 * 分摊策略
 */
export const enum DilutedType {
  /** 不分摊 */
  NotDiluted,
  /** 按电量分摊 */
  QuantityDiluted,
  /** 按商铺面积分摊 */
  AreaDiluted
}

/**
 * 表计类型
 */
export const enum MeterType {
  /** 商户电表 */
  Tenement,
  /** 园区电表 */
  Park,
  /** 公摊电表 */
  Public
}

/**
 * 0.4Kv表计信息
 */
export interface Meter04KvInfo {
  id: string;
  tableKey?: string;
  /** 户址 */
  address: null | string;
  /** 表计表号 */
  meterNo: string;
  /** 条目创建时间，时间格式为 yyyy-MM-dd HH:mm:ss */
  createdAt: string;
  /** 是否可用 */
  enabled: boolean;
  /** 条目最后修改时间，时间格式为 yyyy-MM-dd HH:mm:ss */
  lastModifiedAt: null | string;
  /** 所属园区ID */
  parkId: string;
  /** 表计倍率 */
  ratio: string;
  /** 抄表序号 */
  seq: number;
  /** 表计类型 */
  type: MeterType;
  /** 所辖面积 */
  area: number;
  /** 所在建筑 */
  building: number;
  /** 建筑名称 */
  buildingName: string;
  /** 停用日期 */
  downTime?: string
}

/**
 * 上传Excel后出现错误的信息
 */
export interface UploadExcelError {
  /** 出错的列号 */
  col: number;
  /** 出错信息 */
  error: string;
  /** 出错的行号 */
  row: number;
}

/**
 * 物业附加费信息
 */
export interface MaintenanceInfo {
  /** 条目创建时间，时间格式为 yyyy-MM-dd HH:mm:ss */
  createdAt: string;
  /** 条目删除时间，时间格式为 yyyy-MM-dd HH:mm:ss */
  deletedAt: null | string;
  /** 条目是否启用 */
  enabled: boolean;
  /** 物业附加费所属年份 */
  period: string;
  /** 附加费数额，单位：元/年 */
  fee: string;
  /** 物业附加费记录ID */
  id: string;
  /** 条目最后修改时间，时间格式为 yyyy-MM-dd HH:mm:ss */
  lastModifiedAt: null | string;
  /** 备注 */
  memo: null | string;
  /** 费用项目名称 */
  name: string;
  /** 所属园区ID */
  parkId: string;
}

/** 物业附加费列表条目信息 */
export interface MaintenanceSummary {
  /** 管理单位信息 */
  user: BaseUserInfo;
  /** 管理单位园区信息 */
  park: ParkInfo;
  /** 物业附加费所属年份 */
  period: string;
  /** 合计附加费数额，单位：元/年 */
  fee: string;
  /** 附加费单价， 单位：元/平米/年 */
  price: string;
  /** 附加费单价，单位：元/平米/季 */
  quarterPrice: string;
  /** 附加费单价， 单位：元/平米/半年 */
  semiAnnualPrice: string;
}

/**
 * 园区建筑
 */
export interface ParkBuilding {
  // 是否启用
  enabled: boolean;
  // 建筑楼层数量
  floors: string;
  // 建筑ID
  id: string;
  // 建筑名称
  name: string;
}

/**
 * 更换表计的表单
 */
export interface ExchangeFormData {
  /** 原表计读数 */
  oldReading: MeterReading;
  /** 新表计资料 */
  newMeter: {
    /** 表计编号 */
    code: string;
    /** 倍率 */
    ratio: string;
    /** 表计读数 */
    reading: MeterReading;
  };
}

/**
 * 表计读数
 */
export interface MeterReading {
  /** 有功(尖峰)，非峰谷表读数设为0 */
  critical: string;
  /** 有功(平)，非峰谷表读数设为有功(总)的数值 */
  flat: string;
  /** 有功(峰)，非峰谷表读数设为0 */
  peak: string;
  /** 有功(谷)，非峰谷表读数设为0 */
  valley: string;
  /** 有功(总) */
  overall: string;
}

/**
 * 公摊表计数据
 */
export interface MeterPooled extends Meter04KvInfo {
  /** 关联的子级表计 */
  bindedMeters: Meter04KvInfo[];
}

/**
 * 选择表计数据
 */
export interface MeterChoiceData {
  /** 表计地址 */
  address: string;
  /** 表计编号 */
  code: string;
  /** 标记所属园区ID */
  park: string;
}

/**
 * 商户信息
 */
export interface TenementInfo {
  /** 联系地址 */
  address: string;
  /** 所在建筑ID */
  building: null | string;
  /** 所在建筑名称 */
  buildingName: null | string;
  /** 联系人姓名 */
  contact: null | string;
  /** 条目创建时间 */
  createdAt: string;
  /** 条目删除时间 */
  deletedAt: null | string;
  /** 全名 */
  fullName: string;
  /** 商户ID */
  id: string;
  /** 条目最后修改时间 */
  lastModifiedAt: null | string;
  /** 入住日期 */
  movedInAt: string;
  /** 迁出日期 */
  movedOutAt: null | string;
  /** 所在建筑层数 */
  onFloor: null | string;
  /** 联系电话，将作为主要关联电话使用 */
  phone: null | string;
  /** 简称 */
  shortName: null | string;
}

export interface MeterByTenement {
  /** 园区ID */
  parkId: string,
  /** 商户id */
  tenementId: string,
  /** 电表号 */
  meterId: string,
}


export const enum FeeType {
  HCBN,
  WY,
  WYDS
}

export const FeeTypeList = [
  { label: "华昌宝能收费", value: 0 },
  { label: "物业", value: 1 },
  { label: "物业代收", value: 2 },
]

export interface MeterBox {
  id ?: string,
  park?: string,
  building?: string,
  boxNo?: string,
  address?: string
}

export interface MeterBoxListItem {
  id: string,
  boxNo: string,
  park: ParkInfo,
  building: ParkBuilding,
  enabled: boolean,
  address: string,
  createdAt: string,
  lastModifiedAt: string,
  deletedAt: string,
}