import type { ParkInfo, SimpleParkInfo } from '@/shared/model-park';
import type { BaseUserInfo } from '@/shared/model-user';
import type { Consumption } from '@/shared/model-calculate';
import { Meter04KvInfo, TenementInfo } from '@/shared/model-park';
import { ReportMeterUnit } from '@/shared/model-calculate';

/**
 * 电费核算信息
 */
export interface BasePublicityInfo {
  report: SimpleReportInfo & ReportStatus & { number: string };
  park: SimpleParkInfo;
  user: BaseUserInfo;
}

/**
 * 电费核算报表
 */
export interface SimpleReportInfo {
  /** 报表Id */
  id: string;
  /** 最后一次申请撤回的时间，格式为 yyyy-MM-dd HH:mm:ss */
  lastWithdrawAppliedAt: string;
  /** 最后一次申请审核的时间，格式为 yyyy-MM-dd HH:mm:ss */
  lastWithdrawAuditAt: string;
  /** 所属园区Id */
  parkId: string;
  /** 核算起始日期，格式为 yyyy-MM-dd */
  periodBegin: string;
  /** 核算结束日期，格式为 yyyy-MM-dd */
  periodEnd: string;
  /** 是否已发布 */
  published: boolean;
  /** 发布时间 */
  publishedAt: string;
  /** 报表撤回状态 */
  withdraw: WithdrawState;
}

/**
 * 电费核算报表状态
 */
export interface ReportStatus {
  status: ReportStatusType;
}

/**
 * 电费核算状态
 */
export const enum ReportStatusType {
  /** 数据丢失 */
  Miss_Data,
  /** 数据缺陷 */
  Defects_Data,
  /** 计算错误 */
  Calculation_Data,
  /** 储存错误 */
  Storage_Data,
  /** 除数为0 */
  Divisor_Zero,
  /** 程序错误 */
  Program_Exception,
  /** 成功 */
  Success = 9,
}


export interface CalculateErrorMessage {
  /** 所属报表ID */
  reportId: string;
  /** 出现错误的电表ID */
  meterId: string;
  /** 商户id */
  ID: string;
  /** 商户全称 */
  fullName: string;
  /** 抄表时间 */
  readAt: string;
  /** 电表是否开启 */
  enabled: boolean;
  /** 备注 */
  remark: string;
}

/**
 * 报表的撤回状态
 */
export const enum WithdrawState {
  /** 未撤回 */
  Default,
  /** 申请撤回中 */
  Withdrawing,
  /** 申请拒绝 */
  Refused,
  /** 申请批准 */
  Passed
}

/**
 * 园区核算概况
 */
export interface ParkSummary {
  /** 调整电费 */
  adjustFee: string;
  /** 总建筑面积 */
  area: string;
  /** 基本电费 */
  basicFee: string;
  /** 户表总电量，商户表计+物业表计，不含园区表计和公摊表计 */
  consumption: string;
  /** 线损电量 */
  loss: string;
  /** 线损率，取值为百分数 */
  lossRate: string;
  /** 总消耗电度 */
  overall: Consumption;
  /** 电度摊薄调整电费电价 */
  pooledAdjustFeeByAmount: string;
  /** 面积摊薄调整电费电价 */
  pooledAdjustFeeByArea: string;
  /** 电度摊薄基本电费电价 */
  pooledBasicFeeByAmount: string;
  /** 面积摊薄基本电费电价 */
  pooledBasicFeeByArea: string;
  /** 核算报表ID */
  reportId: string;
  /** 光伏发电电量 */
  generateAmount?: number,
}

/**
 * 公摊表计电费情况
 */
export interface PublicPooled extends Meter04KvInfo {
  /** 接入系统时间，挂表 */
  attachedAt: string;
  /** 从系统移除时间，拆表 */
  detachedAt: null | string;
  /** 所在楼层 */
  onFloor: null | string;
  /** 总电量部分 */
  overall: Consumption;
  /** 摊薄方式，0：按电度摊薄，1：按面积摊薄 */
  poolMethod: PoolMethod;
  /** 表格rowkey标识 */
  tableRowKey: string,
}

/**
 * 摊薄方式: 0:不分摊，1：按电度摊薄，2：按面积摊薄
 */
export const enum PoolMethod {
  No,
  Consumption,
  Area
}

/**
 * 商户电量电费信息
 */
export interface ReportTenement extends TenementInfo {
  consumption: string;
  fee: string;
  pooled: string;
  final: string;
  /** 类型，0-查看明细，1-电费账单 */
  type: 0 | 1;
  /** tabs的key */
  key: string;
  consumptionAmount: string
}

/**
 * 商户详情
 */
export interface ReportTenementDetail {
  park: SimpleParkInfo;
  tenement: TenementInfo;
  comprehensive: TenementComprehensiveDetail;
  meters: (Meter04KvInfo &
    PooledSumDetail &
    ReportMeterUnit & { /** 合计电费 */ finalTotal: string; loss: Consumption } & {
      /** 首次抄表时间 */
      currentTermReadings: string,
      /** 最近抄表时间 */
      lastTermReadings: string,
     }
    & {
    startNumber: number,
    endNumber: number,
    displayRatio: number,
    publicAmount: string,
    refundAmount: string,
  }
     )[];


  pooled: (Meter04KvInfo & ReportMeterUnit)[];
}

/**
 * 商户综合明细
 */
export type TenementComprehensiveDetail = PooledSumDetail & {
  /** 合计电度电量 */
  consumption: string;
  /** 合计电度电费 */
  fee: string;
  /** 电费均价 */
  price: string;
  /** 合计电费 */
  total: string;
  /** 核定线损部分 */
  loss: Consumption;
  /** 摊薄线损电费 */
  basicPooled: string;
  /** 公摊电费 */
  publicPooled: string;
  /** 摊薄调整电费 */
  adjustPooled: string;
  /** 摊薄线损电量 */
  lossAmount: string;
  /** 开始时间 */
  startDate: string;
  /** 结束时间 */
  endDate: string;
  /** 本期公摊电量 */
  publicAmount: number
};

/**
 * 摊薄合计项目
 */
export interface PooledSumDetail {
  /** 摊薄调整电费 */
  adjustPooled: string;
  /** 摊薄基本电费 */
  basicPooled: string;
  /** 摊薄线损电费 */
  lossPooled: string;
  /** 公摊电费 */
  publicPooled: string;
}

/**
 * 公示报表详情
 */
export interface PublicityDetail {
  /** 报表所属用户的简易信息 */
  enterprise: BaseUserInfo;
  /** 报表关联园区信息 */
  park: ParkInfo;
}

/**
 * 终端用户部分全字段
 */
export interface PublicityEndUser {
  /** 地址 */
  address: null | string;
  /** 是否公用也被表计 */
  isPublicMeter: boolean;
  /** 电量(尖峰) */
  critical: string;
  /** 电费(尖峰) */
  criticalFee: string;
  /** 用户名称 */
  customerName: null | string;
  /** 配电物业费 */
  maintenance: string;
  /** 表号 */
  meterId: string;
  /** 电量(总) */
  overall: string;
  /** 电费均价 */
  overallPrice: string;
  /** 电费(总) */
  overallFee: string;
  /** 电量(峰) */
  peak: string;
  /** 电费(峰) */
  peakFee: string;
  /** 电量(谷) */
  valley: string;
  /** 电费(谷) */
  valleyFee: string;
}

/**
 * 终端用户总计部分
 *
 * 终端用户部分
 */
export interface PublicityEndUserSum {
  /** 总电度电费 */
  consumptionFee: string;
  /** 尖峰电量 */
  critical: string;
  /** 尖峰电费 */
  criticalFee: string;
  /** 尖峰电价 */
  criticalPrice: string;
  /** 平电量 */
  flat: string;
  /** 平电费 */
  flatFee: string;
  /** 平电价 */
  flatPrice: string;
  /** 总电量 */
  overall: string;
  /** 电量均价 */
  overallPrice: string;
  /** 峰电量 */
  peak: string;
  /** 峰电费 */
  peakFee: string;
  /** 峰电价 */
  peakPrice: string;
  /** 谷电量 */
  valley: string;
  /** 谷电费 */
  valleyFee: string;
  /** 谷电价 */
  valleyPrice: string;
}

/**
 * 损失总计
 *
 * 损失部分
 */
export interface PublicityLoss {
  /** 电度电费 */
  consumptionFee: string;
  /** 电价 */
  price: string;
  /** 占比 */
  proportion: string;
  /** 电量 */
  quantity: string;
  /** 核定电量 */
  authorizeQuantity: string;
  /** 核定电费 */
  authorizeConsumptionFee: string;
}

/**
 * 调整电费部分
 */
export interface PublicityMaintenance {
  /** 基础电费 */
  basicFees: string;
  /** 调整电费 */
  adjustFee: string;
  /** 损失电费 */
  lossFee: string;
  /** 损失率 */
  lossProportion: string;
  /** 调整电费占比 */
  adjustProportion: string;
  /** 调整电费单价 */
  adjustPrice: string;
}

/**
 * 其他应收合计
 *
 * 其他应收部分
 */
export interface PublicityOthers {
  /** 基础电费 */
  basicFees: string;
  /** 配电运行维护费 */
  maintenanceFee: string;
}

/**
 * 公共总计
 *
 * 公共设施部分
 */
export interface PublicityCategoryOverall {
  /** 总电度电费 */
  consumptionFee: string;
  /** 尖峰电量 */
  critical: string;
  /** 尖峰电费 */
  criticalFee: string;
  /** 尖峰电价 */
  criticalPrice: string;
  /** 平电量 */
  flat: string;
  /** 平电费 */
  flatFee: string;
  /** 平电价 */
  flatPrice: string;
  /** 总电量 */
  overall: string;
  /** 总电费 */
  overallFee: string;
  /** 电量均价 */
  overallPrice: string;
  /** 峰电量 */
  peak: string;
  /** 峰电费 */
  peakFee: string;
  /** 峰电价 */
  peakPrice: string;
  /** 占比 */
  proportion: string;
  /** 谷电量 */
  valley: string;
  /** 谷电费 */
  valleyFee: string;
  /** 谷电价 */
  valleyPrice: string;
}

/** 终端用户指定时间区间内的调整费用统计 */
export interface PublicityEndUserPeriodStatistics {
  /** 用户名称 */
  customerName?: string;
  /** 户址 */
  address?: string;
  /** 表计编号 */
  meterId: string;
  /** 表计是否峰谷表计 */
  pvKind?: number;
  /** 表计是否公共设备表计 */
  isPublicMeter?: boolean;
  /** 总电量 */
  overall: string | null;
  /** 尖峰电量 */
  critical: string | null;
  /** 峰电量 */
  peak: string | null;
  /** 谷电量 */
  valley: string | null;
  /** 总电费 */
  overallFee: string | null;
  /** 尖峰电费 */
  criticalFee: string | null;
  /** 峰电费 */
  peakFee: string | null;
  /** 谷电费 */
  valleyFee: string | null;
  /** 调整费用 */
  adjustFee: string | null;
  /** 调整费用占比 */
  adjustProportion: string | null;
}

/**获取报表核算的错误列表（如果没有返回的是空列表） */
export interface errorMessageItem {
  /** 错误信息备注 */
  remark: string;
  /** 商户id */
  id: string;
  /** 商户全称 */
  fullName: string;
  /** 抄表时间 */
  readAt: string;
  /** 表号 */
  code: string;
  /** 表是否启用 */
  enabled: string;
  /** 报表id */
  rid: string;
}
