//@ts-nocheck
import { SyncParamAction } from '@/shared/foundation';
import { Area } from '@/shared/models';
import { OptionProps } from '@arco-design/web-react/es/Select';
import { notNil } from '@u/funcs';
import {
  filter,
  find,
  includes,
  isEmpty,
  isNil,
  map,
  min,
  not,
  pluck,
  propEq,
  reduce,
  uniq
} from 'ramda';
import { StateSelector } from 'zustand';
import { createStoreHook } from './store_creator';

/**
 * 记录应用中公用的行政区划信息的存储的类型结构。
 */
interface AreaStore {
  areas: Area[];
}

type Actions = {
  /**
   * 向当前应用的行政区划信息存储中加载新的行政区划信息。
   */
  loadAreas: SyncParamAction<Area[]>;
};

/**
 * 应用中公用的行政区划信息存储的初始状态。
 */
const initialAreaStore: AreaStore = {
  areas: []
};

/**
 * 应用中公用的行政区划信息存储。
 */
export const useAreaStore = createStoreHook<AreaStore & Actions>(
  set => ({
    ...initialAreaStore,
    loadAreas: (areas: Area[]) =>
      set(state => {
        state.areas = areas;
      })
  }),
  { debug: false }
);

/**
 * 用于从行政区划信息Store中计算选取所有父级行政区划编号的选择器。
 */
export const allParentCodesSelector: StateSelector<AreaStore, Area[]> = state =>
  uniq(pluck('parent', state.areas));

/**
 * 用于从行政区划信息Store中计算选取所有的顶级行政区划的选择器。
 */
export const topAreasSelector: StateSelector<AreaStore, Area[]> = state => {
  const minLevel = reduce(
    (acc, elem: number) => min(acc, elem),
    Infinity,
    pluck<string, Area>('lev', state.areas)
  );
  return filter(propEq('lev', minLevel), state.areas);
};

/**
 * 基于顶级行政区划选择器进一步形成用于下拉框选项的选择器。
 */
export const topAreaOptionsSelector: StateSelector<AreaStore, OptionProps[]> = state => {
  const parentCodes = allParentCodesSelector(state);
  const topAreas = topAreasSelector(state);
  return map(
    item => ({
      label: item.name,
      value: item.code,
      isLeaf: not(includes(item.code, parentCodes))
    }),
    topAreas
  );
};

/**
 * 用于从行政区划信息Store中计算选取指定行政区划的子级行政区划的选择器。
 */
export const subAreasSelector: (parentCode: string) => StateSelector<AreaStore, Area[]> =
  parentCode => state => {
    return filter(propEq('parent', parentCode), state.areas);
  };

/**
 * 用于将从行政区划信息Store中选出的子级行政区划转换为下拉框使用的选项集的选择器。
 */
export const subAreaOptionsSelector: (
  parentCode: string
) => StateSelector<AreaStore, OptionProps[]> = parentCode => state => {
  const parentCodes = allParentCodesSelector(state);
  return map(
    (item: Area) => ({
      label: item.name,
      value: item.code,
      isLeaf: not(includes(item.code, parentCodes))
    }),
    filter(propEq('parent', parentCode), state.areas)
  );
};

/**
 * 用于生成具有默认值的行政区划级联列表用选项的选择器。
 */
export const topAreaOptionsWithDefaultSelector: (
  codes?: string[]
) => StateSelector<AreaStore, OptionProps[]> = codes => state => {
  const topAreas = topAreaOptionsSelector(state);
  const parentCodes = allParentCodesSelector(state);
  const subAreaOptionsWithDefault = (parent: string) =>
    map<Area, OptionProps>(item => {
      const option: OptionProps = {
        label: item.name,
        value: item.code,
        isLeaf: not(includes(item.code, parentCodes))
      };
      return includes(item.code, parentCodes)
        ? { ...option, children: subAreaOptionsWithDefault(item.code) }
        : option;
    }, filter<Area, Area[]>(propEq('parent', parent), state.areas) ?? []);
  return map(item => {
    return includes(item.value, codes ?? [])
      ? { ...item, children: subAreaOptionsWithDefault(item.value) }
      : item;
  }, topAreas);
};

/**
 * 用于从行政区划信息Store中计算获取指定的行政区划信息用于级联选择的选择器。
 */
export const fullCascaderValueSelector: (
  code: string | null
) => StateSelector<AreaStore, string[]> = code => state => {
  const result: string[] = isEmpty(code) || isNil(code) ? [] : [code];
  let current = find(propEq('code', code), state.areas);
  while (notNil(current)) {
    const parent = find(propEq('code', current.parent), state.areas);
    if (notNil(parent)) {
      result.unshift(parent.code);
    }
    current = parent;
  }
  return result;
};

/**
 * 从行政区划信息Store中获取指定行政区划是否被禁用的选择器。
 */
export const isAreaDisabledSelector: (code: string) => StateSelector<AreaStore, boolean> =
  code => state => {
    const current = find(propEq('code', code), state.areas);
    return current?.disabled ?? false;
  };
