import { SyncAction, SyncParamAction } from '@/shared/foundation';
import { ReactElement, ReactFragment, ReactPortal } from 'react';
import { StateSelector } from 'zustand';
import { createStoreHook } from './store_creator';

export type WorkAreaType = ReactElement | ReactFragment | ReactPortal | HTMLElement;

/**
 * 用于记录应用中所有需要暂存的布局组件引用。
 */
interface LayoutStore {
  /**
   * 用于保存整个工作区的ref，即MainLayout中路由接口之外的部分。
   */
  workAreaRef: WeakRef<WorkAreaType> | null;
  /**
   * 用于保存整个工作区的高度大小。
   */
  workAreaHeight: number;
  /**
   * 页面主面板区域的高度。
   */
  panelHeight: number;
  /**
   * 页面工具栏的高度。
   */
  toolbarHeight: number;
  /**
   * 是否展示覆盖整个页面的遮罩层。
   */
  maskVisible: boolean;
}

type Actions = {
  /**
   * 记录当前工作区组件。
   */
  rememberWorkArea: SyncParamAction<WorkAreaType>;
  /**
   * 清除对于工作区组件的记录。
   */
  forgotWorkArea: SyncAction;
  /**
   * 记录当前工作区组件的高度。
   */
  memorizeWorkAreaHeight: SyncParamAction<number>;
  /**
   * 归零当前工作区组件的高度记录。
   */
  resetWorkAreaHeight: SyncAction;
  /**
   * 记录当前工作区中主要工作面板的高度。
   */
  memorizePanelHeight: SyncParamAction<number>;
  /**
   * 归零当前工作区中主要工作面板高度的记录。
   */
  resetPanelHeight: SyncAction;
  /**
   * 记录当前工作区主要面板中工具栏的高度。
   */
  memorizeToolbarHeight: SyncParamAction<number>;
  /**
   * 归零当前工作区主要工作面板中工具栏高度的记录。
   */
  resetToolbarHeight: SyncAction;
  /**
   * 显示全局遮罩层。
   */
  showMask: SyncAction;
  /**
   * 关闭全局遮罩层的显示。
   */
  hideMask: SyncAction;
};

const initialLayoutState: LayoutStore = {
  workAreaRef: null,
  workAreaHeight: 0,
  panelHeight: 0,
  toolbarHeight: 0,
  maskVisible: false
};

export const useLayoutStore = createStoreHook<LayoutStore & Actions>(
  set => ({
    ...initialLayoutState,
    rememberWorkArea: (workAreaRef: WorkAreaType) =>
      set(state => {
        state.workAreaRef = new WeakRef<WorkAreaType>(workAreaRef);
      }),
    forgotWorkArea: () =>
      set(state => {
        state.workAreaRef = null;
      }),
    memorizeWorkAreaHeight: (height: number) =>
      set(state => {
        state.workAreaHeight = height;
      }),
    resetWorkAreaHeight: () =>
      set(state => {
        state.workAreaHeight = 0;
      }),
    memorizePanelHeight: (height: number) =>
      set(state => {
        state.panelHeight = height;
      }),
    resetPanelHeight: () =>
      set(state => {
        state.panelHeight = 0;
      }),
    memorizeToolbarHeight: (height: number) =>
      set(state => {
        state.toolbarHeight = height;
      }),
    resetToolbarHeight: () =>
      set(state => {
        state.toolbarHeight = 0;
      }),
    showMask: () =>
      set(state => {
        state.maskVisible = true;
      }),
    hideMask: () =>
      set(state => {
        state.maskVisible = false;
      })
  }),
  { debug: false }
);

/**
 * 用于计算页面主面板上主表格的最大高度的选择器。
 */
export const tableHeightSelector: (
  additionalGaps: number,
  adjustHeight: number
) => StateSelector<LayoutStore, number> = (additionalGaps, adjustHeight) => state =>
  state.panelHeight - state.toolbarHeight - 8 * 2 - 8 * additionalGaps - adjustHeight;
