import { SyncAction, SyncParamAction } from '@/shared/foundation';
import {LoginResponse, logout} from '@q/session';
import dayjs from 'dayjs';
import { repeat } from 'ramda';
import { createStoreHook } from './store_creator';
// import {Simulate} from "react-dom/test-utils";
// import reset = Simulate.reset;
import useAsyncFn from "react-use/lib/useAsyncFn";

/**
 * 当前登录的用户信息以及会话信息状态存储类型。
 * ! 注意大部分用户信息已经转移到LocalStorage中存储，这里只保存一些辅助性的无关用户登录状态的内容。
 */
interface SessionStore {
  /**
   * 用于记录用户当前登录即将过期的提示信息是否已经显示过。
   * 三个元素的值分别对应10分钟内提示、5分钟内提示和1分钟内提示。
   * 值为true表示已经显示过，并不再重复显示。
   */
  expireWarning: [boolean, boolean, boolean];
}

type Actions = {
  /**
   * 登出当前用户，重置整个用户状态记录。
   */
  logout: SyncAction;
  /**
   * 处理当前用户登录成功的请求，缓存用户信息。
   */
  login: SyncParamAction<LoginResponse>;
  /**
   * 用于设置哪个阶段的登录会话提示已经显示过了。
   */
  warningDisplayed: SyncParamAction<number>;
};

const initialOpeatorState: SessionStore = {
  expireWarning: [false, false, false]
};

export const useSessionStore = createStoreHook<SessionStore & Actions>(
  set => ({
    ...initialOpeatorState,
    logout: () =>
      set(st => {
        localStorage.clear();
        st.expireWarning = repeat(false, 3) as [boolean, boolean, boolean];
      }),
    login: response =>
      set(st => {
        localStorage.setItem('uid', response.session?.uid ?? '');
        localStorage.setItem('token', response.session?.token ?? null);
        const expiresAt = dayjs(response.session?.expiresAt ?? '2000-01-01 00:00:00').toISOString();
        localStorage.setItem('type', String(response.session?.type ?? -1));
        localStorage.setItem('expires', expiresAt);
        localStorage.setItem('name', response.session?.name ?? 'unknown');
        localStorage.setItem("menu", JSON.stringify(response?.roles || []))
        st.expireWarning = repeat(false, 3) as [boolean, boolean, boolean];
      }),
    warningDisplayed: index =>
      set(st => {
        st.expireWarning[index] = true;
      })
  }),
  { debug: false }
);


type LongTimeNoOperate = {
  /** 有操作，重置 */
  reset: SyncAction;
  /** 无操作，退出 */
  logout: SyncAction
};
export const useLongTimeNoOperate = createStoreHook<{timeStamp: number} & LongTimeNoOperate>(
  set => ({
    timeStamp: Date.now(),
    reset: () => {
      set(st => {
        st.timeStamp = Date.now()
      })
    },
    logout: async () => {
      localStorage.clear();
      set(st => {st.timeStamp = Date.now()})
      const [state, doLogout] = useAsyncFn(logout);
      await doLogout()
    }
  }),
  { debug: false }
);
