//@ts-nocheck
import { Dirtyable, DirtyableValue } from '@/shared/foundation';
import { BaseResponse, Sortable } from '@/shared/model-components';
import BigNumber from 'bignumber.js';
import dayjs from 'dayjs';
import {
  addIndex,
  append,
  assoc,
  assocPath,
  clone,
  compose,
  curry,
  defaultTo,
  equals,
  includes,
  isEmpty,
  isNil,
  keys,
  lens,
  map,
  mergeLeft,
  mergeRight,
  not,
  or,
  path,
  pathEq,
  pathOr,
  prepend,
  prop,
  propEq,
  reduce,
  Reduced,
  set,
  split,
  subtract,
  view,
  __
} from 'ramda';

/**
 * 一个空函数，可以执行，但是不会处理任何事情，主要用于占位
 */
// eslint-disable-next-line @typescript-eslint/no-empty-function
export function noOp() {}

/**
 * 判断给定的参数是否不是undefined和null。
 */
export const notNil = compose(not, isNil);

/**
 * 判断给定的字段是否为空。
 */
export const propIsNil = curry(<T>(property, obj: T) => compose(isNil, prop(property))(obj));

/**
 * 判断给定的字段是否不为空。
 */
export const propNotNil = curry(<T>(property, obj: T) => compose(notNil, prop(property))(obj));

/**
 * 判断给定的参数是否不是含义上的空，例如空字符串、空列表等。
 */
export const notEmpty = compose(not, isEmpty);

/**
 * 判断给定的两个参数是否不等。
 */
export const notEquals = curry(compose(not, equals));

/**
 * 判断对象中给定的属性是给定值之间是否不相等。
 */
export const notPropEq = curry(compose(not, propEq));

/**
 * 判断给定的值是否为null、未定义或者空白。
 * @param value 要判断是否可能为空或者空白的值
 * @returns 值的空白判断结果。
 */
export const isNilOrEmpty = <T>(value: T | null | undefined): boolean => or(isNil(value), isEmpty(value));

/**
 * 判断给定的值是否既不是null、未定义，也不是空白。
 */
export const norNilOrEmpty = compose(not, isNilOrEmpty);

/**
 * 用于搭配map方法更改数组中对象的键名，或者可以直接使用`renameKeys({ oldKey: newKey })(obj)`的形式修改某一对象中的属性名。
 */
export const renameKeys = curry((keysMap: object, obj) =>
  reduce((acc, key) => assoc(keysMap[key] || key, prop(key, obj), acc), {}, keys(obj))
);

/**
 * 用于将对象中指定的键的内容中指定键的内容转换为普通数字表示。如果指定的键不是精确数据类型，那么将自动跳过。
 * @param includeKeys 指定需要转换的键。
 * @param obj 需要转换的对象。
 */
export const convertToNumber = curry(<T>(includeKeys: string[], obj: T) =>
  reduce(
    (acc: any, key: string | number | symbol) => {
      const value = prop(key, obj);
      return assoc(key, includes(key, includeKeys) ? new BigNumber(value).toNumber() : value, acc);
    },
    {},
    keys(obj)
  )
);

/**
 * 用于将对象中指定的键的内容转换为使用BigNumber的精确数字表示。如果指定时的键不是数字类型，那么将自动跳过。
 * @param includeKeys 指定需要转换的键。
 * @param obj 需要转换的对象。
 */
export const convertToPreciseness = curry(<T>(includeKeys: (string | number | symbol)[], obj: T) =>
  reduce(
    (acc: any, key: string | number | symbol) => {
      const value = prop(key, obj);
      return assoc(
        key,
        includes(key, includeKeys) && not(new BigNumber(value).isNaN()) ? new BigNumber(value) : value,
        acc
      );
    },
    {},
    keys(obj)
  )
);

/**
 * 用于定义用在`JSON.parse()`值进行转换的柯里化函数。
 * @param key 对象中的键
 * @param value 对象中的键对应的值
 * @param transformKeys 定义需要转换的键
 * @returns 执行转换的柯里化函数。
 */
const parseBigNumber = curry((key: string | number | symbol, value: any, transformKeys: (string | number | symbol)[]) =>
  includes(key, transformKeys) && not(new BigNumber(value).isNaN()) ? new BigNumber(value) : value
);

/**
 * 用于在`JSON.parse()`方法中将指定的键值转换为使用BigNumber表示的精确数字的方法。
 * @param transformKeys 定义需要转换的键
 * @returns 用于进行BigNumber类型转换的函数。
 */
export const parseWithPreciseness = (transformKeys: (string | number | symbol)[]) =>
  parseBigNumber(__, __, transformKeys);

/**
 * 用于定义用在`JSON.stringify()`值进行转换的柯里化函数。
 * @param key 对象中的键
 * @param value 对象中的键对应的值
 * @param  transformKeys 定义需要转换的键
 * @returns 执行转换的柯里化函数。
 */
const stringifyBigNumber = curry(
  (key: string | number | symbol, value: any, transformKeys: (string | number | symbol)[]) =>
    includes(key, transformKeys) ? new BigNumber(value).toNumber() : value
);

/**
 * 用于在`JSON.stringify()`方法中将指定的键值转换为普通数字表示的方法。
 * @param transformKeys 定义需要转换的键
 * @returns 用于进行BigNumber类型转换的函数
 */
export const stringifyWithPreciseness = (transformKeys: (string | number | symbol)[]) =>
  stringifyBigNumber(__, __, transformKeys);

/**
 * 用于在组件中渲染输出带有计价单位的数字。
 * ! 之前用于适配Acro Design，待适配Ant Design。
 * @param col 表格组件所取得的列值
 * @param record 表格组件当前行所使用的数据原值
 * @returns 由数字与计价单位拼合起来的展示文字
 */
export const renderNumberWithUnit = (unit?: string) => (col: number | null | undefined, record: any) =>
  isNil(col) ? '-' : `${col} ${pathOr('', split('.', unit ?? 'chargeUnit'), record)}`;

/**
 * 输出一个可空数字值，如果值为空，则返回-。
 * @param precision 输出数字的精度，默认精度为2。
 * @returns 可空数字的输出字符串
 */
export const renderNullableNumber =
  (precision = 2) =>
  (col: number | null, record: any) =>
    isNil(col) ? '-' : new BigNumber(col).toFixed(precision);

/**
 * 用于在组件中渲染输出带有货币单位的数字。
 * ! 之前用于适配Acro Design，待适配Ant Design。
 * @param currency 要使用的货币符号
 * @param precision 展示使用的数字精度
 * @returns 由货币符号和货币金额组合起来的展示文字生成函数
 */
export const renderCurrency =
  (currency: string, precision = 2) =>
  (col: number, record?: any) =>
    isNil(col) ? '-' : `${currency}${new BigNumber(col).toFixed(precision)}`;

/**
 * 输出带有货币单位的数字。
 * @param amount 货币金额
 * @param currency 货币符号
 * @param precision 货币精度，小数点位数
 * @param groupped 是否按千位分组
 * @returns 由货币符号和货币金额组合起来的展示文字
 */
export const renderGroupedCurrency = (
  amount: number | null,
  currency: string,
  precision = 2,
  groupped = true
): string => {
  const precisedAmount = new BigNumber(amount ?? 0);
  const fmt = {
    prefix: `${currency} `,
    decimalSeparator: '.'
  };
  const grouppedFmt = mergeRight(fmt, { groupSeparator: ',', groupSize: 3 });
  return isNil(amount)
    ? '-'
    : groupped
    ? precisedAmount.toFormat(precision, grouppedFmt)
    : precisedAmount.toFormat(precision, fmt);
};

/**
 * 用于在组件中渲染输出指定精确度百分比的数字。
 * ! 之前用于适配Acro Design，待适配Ant Design。
 * @returns 由指定精确度百分比和百分号组合起来的展示文字
 */
export const renderPercent =
  (presicision = 2) =>
  (col: number, _?: unknown) =>
    isNil(col) ? '-' : `${new BigNumber(col).times(100).toFixed(presicision)} %`;

/**
 * 对从服务端获取到的响应数据进行错误提示输出。
 * @param tag 错误输出的标签信息
 * @param response 从服务端获取到的响应数据
 * @param error 可能伴随存在的错误。
 */
export const logError = (tag: string, response: BaseResponse, error?: Error) => {
  console.error(`交互时异常[${tag}]:`, response.code, response.message);
  if (notNil(error)) {
    console.error(error);
  }
};

/**
 * 对系统在运行过程中出现的异常进行错误提示输出。
 * @param tag 错误输出的标签信息。
 * @param error 可能伴随存在的错误。
 */
export const logException = (tag: string, error?: any) => {
  console.error(`运行时异常[${tag}]: `, notNil(error) ? errorMessage(error) : '未知错误');
};

/**
 * 用于从携带有错误信息的错误对象实例中获取错误文字描述信息。
 * @param error 错误对象实例
 * @returns 错误文字描述信息
 */
export const errorMessage = prop<'message', string>('message');

type ResponseTransformer<T extends BaseResponse> = (response: string) => T;

/**
 * 用于对从服务端返回的响应数据结构进行变形处理，主要用于生成一个用于变形处理的函数。
 *
 * * 用于存储变形数据的键是不可以改名的。
 *
 * @param nameMap 用于在改变数据结构时，键名称的替换。
 * @param extractKeys 用于提取存储变形数据的键。
 * @param collection 需要变形的数据是否是一个集合。
 * @returns 返回一个用于变形处理的函数。
 */
export const responseTransformer: <T extends BaseResponse>(
  nameMap: Record<string, string>,
  extractKeys: string[],
  collection: boolean
) => ResponseTransformer<T> = (
  nameMap: Record<string, string>,
  extractKeys: string[],
  collection: boolean
): ResponseTransformer<T> => {
  return (response: string) => {
    const data = JSON.parse(response);
    const baseResponse = {
      code: prop('code', data),
      message: prop('message', data)
    };
    const transformedResult = reduce(
      (acc, elem) => {
        const result = collection
          ? map(renameKeys(nameMap, __), prop(elem, data) ?? [])
          : renameKeys(nameMap, prop(elem, data) ?? {});
        return { ...acc, [elem]: result };
      },
      {},
      extractKeys
    );
    return mergeRight(baseResponse, transformedResult);
  };
};

/**
 * 用于将一个对象转换成其中的每个字段都使用可脏数据类型包装的对象，所有字段默认都是不脏的。
 * @param original 原始对象
 * @returns 转换后的可脏对象
 */
export const convertToDirtyable = <T extends object>(original: T): Dirtyable<T> => {
  return reduce(
    (acc, elem) =>
      mergeRight({ [elem]: { value: prop(elem, original), lastValue: prop(elem, original), dirty: false } }, acc),
    {},
    keys(original)
  );
};

/**
 * 用于对两个日期对象进行比较，返回可以被比较函数使用的数字值。
 * @param date1 第一个日期对象
 * @param date2 第二个日期对象
 * @returns 两个日期的比较结果
 */
export const compareDate = (date1: Date, date2: Date) => {
  const date1Time = dayjs(date1);
  const date2Time = dayjs(date2);
  return date1Time.isBefore(date2Time) ? -1 : date1Time.isAfter(date2Time) ? 1 : 0;
};

/**
 * 翻译税模式的值到税模式的文字描述。
 * @param mode 税模式的值
 * @returns 税模式的文字描述
 */
export const translateTaxMode = (mode: string): string => {
  return defaultTo('未知', { '0': '未税', '1': '含税' }[mode]);
};

/**
 * 生成聚焦可脏对象中某个可脏属性的聚焦函数。
 * @param property 要聚焦的可脏对象中的属性名。
 * @returns 分别用于聚焦可脏对象中的三个属性的聚焦函数
 */
export const lensDirtyable = (property: string) => {
  return [
    lens(path([property, 'value']), assocPath([property, 'value'])),
    lens(path([property, 'dirty']), assocPath([property, 'dirty'])),
    lens(path([property, 'lastValue']), assocPath([property, 'lastValue']))
  ];
};

/**
 * 用于直接更改一个可脏类型（通常是基础类型）的值，原有值将自动被赋予新可脏实例的lastValue属性，同时整个可脏对象变为脏对象。
 * * 将会直接返回一个可脏对象的新实例。
 * @param value 可脏类型的的新值。
 * @param origin 可脏类型对象的原始实例。
 * @returns 所携带的值已经被更新的可脏对象的新实例。
 */
export const setDirtyableValue = <T>(value: T, origin: DirtyableValue<T>): DirtyableValue<T> => {
  return {
    dirty: true,
    lastValue: origin.value,
    value
  };
};

/**
 * 直接更新一个可脏对象中的某个属性的值，将自动将属性的原有值赋予lastValue记录，同时整个对象变为脏对象。
 * * 将会直接返回一个可脏对象的新实例。
 * @param property 要修改的属性名称。
 * @param value 指定属性的新值。
 * @param origin 可脏对象的原始实例。
 * @returns 可脏对象被修改以后的新的已脏实例。
 */
export const setDirtyableProperty = <T>(property: string, value: unknown, origin: Dirtyable<T>): Dirtyable<T> => {
  const [lensValue, lensDirty, lensLastValue] = lensDirtyable(property);
  let newObj = set(lensDirty, true, origin);
  newObj = set(lensLastValue, view(lensValue, origin), newObj);
  newObj = set(lensValue, value, newObj);
  return newObj;
};

/**
 * 检测指定的可脏对象是否包含脏属性。
 * @param dirtyable 要检测的可脏对象。
 * @param properties 可选，如果设置了该参数，则只检查该参数指定的属性是否脏。
 * @returns 给定的可脏对象是否包含脏属性。
 */
export const isDirtied = <T>(dirtyable: Dirtyable<T>, properties?: string[]): boolean => {
  return reduce(
    (acc, e) => acc || pathEq([e, 'dirty'], true, dirtyable),
    false,
    isNil(properties) ? keys(dirtyable) : properties
  );
};

/**
 * 进行一个预判断的右向合并操作，如果给定的条件不成立，那么将不执行合并，直接返回a对象的克隆；
 * 如果给定的条件成立，那么将会将b对象合并进入a对象中。
 * @param predicate 是否执行合并的判断条件。其中可以接受的第一个参数是合并的对象，第二个参数是基底对象。
 * @param a 用于合并的基底对象。
 * @param b 用于合并的新对象，将会覆盖基底对象中相同的键。
 * @returns 完成合并的新对象，或者原始的基底对象的克隆。
 */
export const mergeRightBy = <T>(
  predicate: (origin?: Partial<T>, target?: Partial<T>) => boolean,
  a: Partial<T>,
  b: Partial<T>
): Partial<T> => {
  if (predicate(b, a)) {
    return mergeRight(a, b);
  } else {
    return clone(a);
  }
};

/**
 * 进行一个预判断的左向合并操作，如果给定的条件不成立，那么将不执行合并，直接返回b对象的克隆；
 * 如果给定的条件成立，那么将会将a对象合并进入b对象中。
 * @param predicate 是否执行合并的判断条件。其中可以接受的第一个参数是合并的对象，第二个参数是基底对象。
 * @param a 用于合并的对象，将会覆盖基底对象中相同的键。
 * @param b 用于合并的基底对象。
 * @returns 完成合并的新对象，或者是原始的基底对象的克隆。
 */
export const mergeLeftBy = <T>(
  predicate: (origin?: Partial<T>, target?: Partial<T>) => boolean,
  a: Partial<T>,
  b: Partial<T>
): Partial<T> => {
  if (predicate(a, b)) {
    return mergeLeft(a, b);
  } else {
    return clone(b);
  }
};

/**
 * 进行一个预判断的列表追加操作，如果给定的条件不成立，那么将不执行追加，直接返回原列表的克隆；
 * 如果给定的条件成立，那么将会将a对象追加进入列表b中，并返回追加后列表的新实例。
 * @param predicate 是否执行追加的判断条件，其中可以接受的第一个参数是即将被追加的元素本身，第二个参数是列表本身。
 * @param a 用于追加到列表中的元素。
 * @param b 用于存放数据的列表。
 * @returns 完成追加的新列表对象或者原始列表对象的克隆。
 */
export const appendBy = <T>(
  predicate: (origin?: Partial<T>, target?: Partial<T>[]) => boolean,
  a: Partial<T>,
  b: Partial<T>[]
): Partial<T>[] => {
  if (predicate(a, b)) {
    return append(a, b);
  } else {
    return clone(b);
  }
};

/**
 * 进行一个预判断的列表头部添加操作，如果给定的条件不成立，那么将不执行添加，直接返回原列表的克隆；
 * 如果给定的条件成立，那么将会将a对象添加进入列表b的头部，并返回添加后列表的新实例。
 * @param predicate 是否执行添加的判断条件，其中可以接受的第一个参数是即将被添加的元素本身，第二个参数是列表本身。
 * @param a 用于添加到列表中的元素。
 * @param b 用于存放数据的列表。
 * @returns 完成添加的新列表对象或者原始列表对象的克隆。
 */
export const prependBy = <T>(
  predicate: (origin?: Partial<T>, target?: Partial<T>[]) => boolean,
  a: Partial<T>,
  b: Partial<T>[]
): Partial<T>[] => {
  if (predicate(a, b)) {
    return prepend(a, b);
  } else {
    return clone(b);
  }
};

/**
 * 确认对象中指定键的值是否包含在可取值列表中。
 * @param a 指定用于筛选的对象的键。
 * @param b 指定对象的键可取值列表。
 * @returns 对象是否符合包含条件。
 */
export const includeByProp = <T, P extends keyof T>(a: P, b: T[P][]): boolean =>
  compose<T, boolean>(includes<T[P]>(__, b), prop<T>(a));

/**
 * 确认对象中指定键的值是否不包含在指定值的列表中。
 * @param a 指定用于筛选对象的键。
 * @param b 指定对象键的不可取值列表。
 * @returns 对象是被不符合包含条件。
 */
export const notIncludeByProp = <T, P extends keyof T>(a: P, b: T[P][]): boolean =>
  compose<T, boolean>(not, includeByProp(a, b));

/**
 * 对两个可排序对象进行减法排序，直接调用可产生升序排序效果。
 * @param a 参与排序的可排序对象1。
 * @param b 参与可排序对象的对象2。
 * @returns 对象1与对象2的排序字段相减的结果。
 */
export const subtractSortable = <T extends Sortable>(a: T, b: T): number => subtract(a.sort, b.sort);

type ReduceIndexedType = <T, R>(
  fn: (acc: R, elem: T, index: number, list: readonly T[]) => R | Reduced<R>,
  acc: R,
  list: readonly T[]
) => R;

/**
 * 带索引的Reduce函数。其变形函的参数依次为acc、elem、index、list。
 */
export const reduceIndexed: ReduceIndexedType = addIndex<T, R>(reduce<T, R>);

/**
 * 用于对有精度要求的数字进行整理和输出
 */
export const outputWithPrecision: (
  origin: string | number | null | undefined,
  dp: number,
  defaultValue: string
) => string = (origin, dp, defaultValue): string => {
  if (isNilOrEmpty(origin)) {
    return defaultValue;
  }
  if (not(isFinite(dp))) {
    return new BigNumber(origin).toString();
  }
  return new BigNumber(origin).toFixed(dp, BigNumber.ROUND_HALF_EVEN);
};

/**
 * 将数字金额转为大写
 */
export const digitUppercase = (n: number) => {
  const fraction = ['角', '分'];
  const digit = ['零', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖'];
  const unit = [
    ['元', '万', '亿'],
    ['', '拾', '佰', '仟']
  ];
  n = Math.abs(n);
  let s = '';
  for (let i = 0; i < fraction.length; i++) {
    s += (digit[Math.floor(n * 10 * Math.pow(10, i)) % 10] + fraction[i]).replace(/零./, '');
  }
  s = s || '';
  n = Math.floor(n);
  for (let i = 0; i < unit[0].length && n > 0; i++) {
    let p = '';
    for (let j = 0; j < unit[1].length && n > 0; j++) {
      p = digit[n % 10] + unit[1][j] + p;
      n = Math.floor(n / 10);
    }
    s = p.replace(/(零.)*零$/, '').replace(/^$/, '零') + unit[0][i] + s;
  }
  return s
    .replace(/(零.)*零元/, '元')
    .replace(/(零.)+/g, '零')
    .replace(/^整$/, '零元');
};
